<?php

	/*=======================================================================
	 * copyright (c) AL-Design Works 
	 * http://cms.al-design.jp/
	 * このソースファイルは CMS Designer の一部です。
	 * 詳しいライセンス規約についてはウェブサイトを参照してください。
	 *=======================================================================
	 */

	require_once( dirname(__FILE__) . "/app.php.inc" );
	
	/* ---------------------------------------------------------------
	 * ページキャッシュ管理クラス。
	 * クラスメソッドのみ使用可能
	 */
	
	// ページキャッシュのデフォルト設定値（全ページに有効。但し個別ページの設定が優先される。）
	define( 'AD_PAGECACHE_ENABLED', false ); // false : キャッシュしない,  true : キャッシュする
	define( 'AD_PAGECACHE_EXPIRES', '' ); // '' : キャッシュ有効期限なし,  '時:分' : 毎日その時刻を超えたらキャッシュを再作成する。例）'5:00'
	
	$gADPageCacheEnabled = AD_PAGECACHE_ENABLED;
	function cmsd_set_pagecache_enabled( $enabled )
	{
		global $gADPageCacheEnabled;
		$gADPageCacheEnabled = ( $enabled === true );
	}
	
	function cmsd_get_pagecache_enabled()
	{
		global $gADPageCacheEnabled;
		return $gADPageCacheEnabled;
	}
	
	$gADPageCacheExpires = AD_PAGECACHE_EXPIRES;
	function cmsd_set_pagecache_expires( $expires )
	{
		global $gADPageCacheExpires;
		$gADPageCacheExpires = $expires;
	}
	
	function cmsd_get_pagecache_expires()
	{
		global $gADPageCacheExpires;
		return $gADPageCacheExpires;
	}
	
	function cmsd_pagecache( $param )
	{
		if ( $param === "on" ) 
		{
			cmsd_set_pagecache_enabled( true );
		}
		else if ( $param === "off" ) 
		{
			cmsd_set_pagecache_enabled( false );
		}
		else {
			// H:M形式のチェック
			if ( ! preg_match( '/^\d{1,2}:\d{1,2}$/', $param ) )
			{
				cmsd_abort_proc( "cmsd_pagecache() function must be given 'on'|'off'|H24:M in the parameter." );
			}
			
			// 時刻として妥当な値か？
			$hour = '';
			$minute = '';
			list( $hour, $minute ) = explode( ':', $param );
			
			$hour = intval( $hour );
			$minute = intval( $minute );
			
			if ( $hour < 0 || $hour > 23 || $minute < 0 || $minute > 59 )
			{
				cmsd_abort_proc( "cmsd_pagecache() : '$param' is not valid time string(H24:M)." );
			}
			
			// グローバル変数に格納
			cmsd_set_pagecache_enabled( true );
			cmsd_set_pagecache_expires( $param );
		}
	
	}
	
	class ADPageCache
	{
		/// クラスメソッド群
		public static function isEnabled()
		{
			return cmsd_get_pagecache_enabled();
		}
		
		// リクエストURLからキャッシュIDを生成する
		public static function createCacheIDFromRequest()
		{
			$server_name = $_SERVER['SERVER_NAME'];
			$script_name = $_SERVER['SCRIPT_NAME'];
			$query_string = isset($_SERVER['QUERY_STRING']) ? $_SERVER['QUERY_STRING'] : '';
			
			$cacheID = $server_name . $script_name . $query_string;
			
			// 先頭のスラッシュ(/)を削除
			//$cacheID = substr( $script_name . $query_string, 1 );
			
			// セッションID部分を除去
			$sessionName = session_name();
			$expstr = '/(&|)' . $sessionName . '=[^&]*/';
			$cacheID = preg_replace( $expstr, "", $cacheID );
			
			// ファイル名として使えない文字を置換（ピリオドは、不正な拡張子対策）
			$cacheID = strtr( $cacheID, '\/:?*"><.', '__________' );
			
			// プレフィクス"pg"をつける
			$cacheID = "pg" . $cacheID;
			
			// 250文字以上なら、空文字を生成（つまり、キャッシュを生成しない）
			if ( strlen( $cacheID ) > 250 )
			{
				$cacheID = "";
			}
			
			return $cacheID;
		}
		
		// キャッシュデータの存在チェック
		public static function exists( $cacheID )
		{
			// キャッシュIDが空文字の場合、キャッシュなし
			if ( $cacheID == "" ) return false;
			// ファイル存在チェック
			return file_exists( ADPageCache::getCacheFilePath( $cacheID ) );
		}
		
		// キャッシュデータのファイルパス
		public static function getCacheFilePath( $cacheID )
		{
			return App::getCacheDir() . "pagecache/" . $cacheID;
		}
		
		// キャッシュデータを標準出力へ出力する
		public static function outputCache( $cacheID )
		{
			$filename = ADPageCache::getCacheFilePath( $cacheID );
		
			$fp = fopen( $filename, 'rb');
			flock( $fp, LOCK_SH );	// 読み込みロック（共有ロック）
			while (!feof($fp))
			{
				echo fread($fp, 4096);
			}
			flush();
			@flock( $fp, LOCK_UN );
			fclose($fp);
		
			//readfile( ADPageCache::getCacheFilePath( $cacheID ) );
		}
		
		// キャッシュデータを生成する（存在する場合は上書きする）
		public static function createCache( $cacheID, $pagedata )
		{
			if ( $cacheID != "" )
			{
				$filename = ADPageCache::getCacheFilePath( $cacheID );
				$fp = @fopen( $filename, 'w' );
				if ( $fp )
				{
					// 書き込みロックに失敗したらキャッシュを作成しない。キャッシュがない場合は無条件で作成
					if ( ! ADPageCache::exists( $cacheID ) || flock( $fp, LOCK_EX ) )	
					{
						@fwrite( $fp, $pagedata );
						flock( $fp, LOCK_UN );
					}
					
					@fclose( $fp );
					@chmod( $filename, 0666 );
				}
			}
		}
		
		/*
		// キャッシュデータが存在しない場合のみ、キャッシュデータを生成する。
		function createCacheIfNoCache( $cacheID, $pagedata )
		{
			if ( ! ADPageCache::exists( $cacheID ) )
			{
				ADPageCache::createCache( $cacheID, $pagedata );
			}
		}
		*/
		
		// 全キャッシュファイルを削除
		public static function deleteAllCaches()
		{
			$pageCacheFilePath = App::getCacheDir() . "pagecache/*";
			$files = glob( $pageCacheFilePath );
			if ( $files )
			{
				foreach( $files as $filename )
				{
					unlink( $filename );
				}
			}
		}
		
		// キャッシュが有効期限切れか？（有効期限切れならtrueを返す）
		public static function isExpired( $cacheID )	// $expires = '7:30'
		{
			// 更新予定時刻が指定されていなければ、有効期限切れなし。
			$expires = cmsd_get_pagecache_expires();
			if ( $expires === "" )
			{
				return false;	
			}
			
			// キャッシュファイルが存在しなければ、有効期限切れを返す
			if ( ! ADPageCache::exists( $cacheID ) )
			{
				return true;
			}
		
			// キャッシュファイルの作成日時
			$timeOfCache = filemtime( ADPageCache::getCacheFilePath( $cacheID ) );
			
			// 現在日時
			$timeOfNow = time();
			
			// ファイル作成日時からの経過時間
			$timeDiffPast = $timeOfNow - $timeOfCache;
			
			// 更新予定時刻（ローカル）からタイムスタンプを生成（日付は現在のものを使用）
			$updateHour = '';
			$updateMinute = '';
			
			
			list( $updateHour, $updateMinute ) = explode( ':', $expires );
			$timeOfUpdate = mktime( $updateHour, $updateMinute );
			
			// ファイル作成時刻から更新予定時刻までの猶予時間
			$timeDiffRemain = (( $timeOfUpdate + 60 * 60 * 24 ) - $timeOfCache ) % ( 60 * 60 * 24 );
			
			if ( $timeDiffPast > $timeDiffRemain )
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
	}
?>