<?php

	/*=======================================================================
	 * copyright (c) AL-Design Works 
	 * http://cms.al-design.jp/
	 * このソースファイルは CMS Designer の一部です。
	 * 詳しいライセンス規約についてはウェブサイトを参照してください。
	 *=======================================================================
	 */

	require_once( dirname(__FILE__) . "/app.php.inc" );
	require_once( dirname(__FILE__) . "/xmlutil.php.inc" );
	require_once( dirname(__FILE__) . "/user.php.inc" );	// 2005.07.28 追加。非公開データの閲覧の為。
	//require_once( dirname(__FILE__) . "/viewtag.php.inc" );	// 2005.08.25 追加。
	//require_once( dirname(__FILE__) . "/cmsdconfig.php.inc" );	// 2005.09.22 追加。→2015.08.27 app.php.incに移動

	// セッション開始
	//session_start();

	function dispview( $name, $kind )
	{
		// エントリIDを決定する。
		$id = $kind;	// ID用の変数。$kindが"index"か"recentry"の場合はそのまま使う
		if ( $kind == "entry" )
		{
			// "entry"の場合はリクエストからIDを取得する。
			$id = $_REQUEST["e"];		// ユーザ入力"e"を取得。
		}
		
		// HTMLの生成
		$v = new View( $name );
		$v->renderHTML( $id );
	}
	
	
	// スキーマ情報を配列で返す。（リスト項目の内部項目も同列に返す）
	// ex ) schemaInfoArr[ "data1" ][ "type" ]
	function getSchemaInfoByArray( $schemaName )
	{
		// 結果arrayの生成
		$array = array();
	
		// スキーマDOMの取得
		$xmlDocSchema = domxml_open_file( App::createSchemaFilePath( $schemaName ) );
		$xpathSchema = $xmlDocSchema->xpath_new_context();
		
		$xmlSchemaDataArr = $xpathSchema->xpath_eval( "/schema//data" );
		
		foreach ( cmsd_get_nodeset( $xmlSchemaDataArr ) as $xmlSchemaData )
		{
			$name = $xmlSchemaData->get_attribute( 'name' );
			$attributes = $xmlSchemaData->attributes();
			
			$schemaInfo = array();
			foreach ( $attributes as $attribute )
			{
				$schemaInfo[ $attribute->name() ] = $attribute->value();
			}
			
			// メニュー型の項目の場合、子要素であるmenuitem要素も取得する
			if ( $schemaInfo['type'] === 'menu' )
			{
				// menuitem要素の読み込み
				$xmlMenuItemList = $xmlSchemaData->get_elements_by_tagname( 'menuitem' );
				$menuItems = array();
				foreach ( $xmlMenuItemList as $xmlMenuItem )
				{
					$menuItems[ file2code( $xmlMenuItem->get_attribute( 'id' ) ) ] = file2code( $xmlMenuItem->get_content() );
				}
				$schemaInfo['menuItems'] = $menuItems;
			}
			
			$array[ $name ] = $schemaInfo;
		}
		
		return $array;
		
	}
	
	
	class cmsview_common
	{
		public static function IsShowingClosedEntry()
		{
			return ( isset( $_GET['showclosedentry'] ) && $_GET['showclosedentry'] == 'yes' );
		}
	
		/* ---------------------------------------------------------------
		 * 指定したentryのスキーマ名をSite.Configより取得する。
		 * $entryName : エントリ名。
		 */
		public static function getSchemaName( $entryName )
		{
			// Site.Configより指定されたエントリ設定情報を取得する。
			$xmlDocSite = domxml_open_file( App::getSiteConfigFilePath() );
			$xpath = $xmlDocSite->xpath_new_context();
			
			$xmlEntryConfig = $xpath->xpath_eval( "//entry[@name='" . $entryName . "']" );
			
			// 2005.7.7 追加
			if ( count( cmsd_get_nodeset( $xmlEntryConfig ) ) <= 0 )
			{
				//trigger_error( "site.config.xmlファイルに'$entryName'のエントリ定義がありません。", E_USER_ERROR );
				cmsd_abort_proc( "site.config.xmlファイルに'$entryName'のエントリ定義がありません。");
			}
			
			$schemaName = $xmlEntryConfig->nodeset[0]->get_attribute( "schema" );
			
			// 2005.7.7 追加
			if ( $schemaName == '' )
			{
				cmsd_abort_proc( "site.config.xmlファイルの'$entryName'のエントリ定義にschema属性が指定されていません。" );
			}
			
			return $schemaName;
		}
		
		
		public static function createXPathCondition( $groupKeyValue, $publishControl )
		{
			// 公開フラグ、および公開期間の条件生成
			$condForPublish = cmsview_common::createXPathConditionForPublish( $publishControl );
			
			// グループによる絞込み
			if ( empty($groupKeyValue))
			{
				// グループキーが指定されていなければ、全て取得する。
				/// 公開条件が存在すれば、[]で囲む。
				if ( strlen($condForPublish) > 0 )
				{
					$cond = '[' . $condForPublish . ']';
				}
				else
				{
					$cond = '';
				}
			}
			else
			{
				// グループキーが指定されていたら、グループで絞り込む。
				$condForGroup = cmsview_common::createXPathConditionForGroup( $groupKeyValue );
				if ( strlen($condForGroup) > 0 )
				{
					if ( strlen($condForPublish) > 0 )
					{
						$cond = '[' . $condForPublish . ' and ' . $condForGroup . ']';
					}
					else
					{
						$cond = '[' . $condForGroup . ']';
					}
				}
				else
				{
					if ( strlen($condForPublish) > 0 )
					{
						$cond = '[' . $condForPublish . ']';
					}
					else
					{
						$cond = '';
					}
				}
			}
			
			//print htmlentities( $cond );
			return $cond;
		}
		
		public static function createXPathConditionForPublish( $publishControl )
		{
			// 年月日時分
			$sYear = sprintf( "%04d", date('Y') );
			$sMonth = sprintf( "%02d", date('m') );
			$sDay = sprintf( "%02d", date('d') );
			$sHour = sprintf( "%02d", date('H') );
			$sMinute = sprintf( "%02d", date('i') );

			//$strToday = $sYear . '-' . $sMonth . '-' . $sDay . ' ' . $sHour . ':' . $sMinute;
			$strToday = $sYear . $sMonth . $sDay . $sHour . $sMinute;
			
			// 「公開」かつ、開始日時≦現在日時≦終了日時 のものを抽出する条件式。
			// 公開フラグや開始日時、終了日時が存在しなかった頃のバージョンとの互換性の為に、それらが無いエントリも考慮した条件式にする。
			// ※XPathにて文字列の大小比較が出来ない事が判明。但し、数値文字列の場合のみ比較可能だったので、:や-などを外して比較することにした。
			
			//$cond = "( not(@visible) or @visible='True' )";
			//$cond = "(not( @visible='False' or ( @visible='True' and (( @startdate >= '$strToday' ) or ( @enddate <= '$strToday' )) )))";
			
			// 公開方法によって条件式を変更
			
			switch ( $publishControl )
			{
				// 全て表示
				case 'none' :
					$cond = '';
					break;
				
				// 公開・非公開のみ
				case 'on-off' :
					$cond = "not(@visible='False')";
					break;
				
				// タイマー公開も使う
				case 'full' :
					$cond = "(not( @visible='False' or ( @startdate > '$strToday' ) or ( @enddate <= '$strToday' )))";
					break;
			}
			
			// 非公開エントリの強制表示処理
			// ※showclosedentryパラメータが"yes"で且つ、コンテンツ管理画面にログイン済の場合は、全て公開とする。
			if ( cmsview_common::IsShowingClosedEntry() and LoginManager::IsAuthed() )
			{
				$cond = "";
			}
			
			return $cond;
		}
		
		public static function escapeStringInXPath( $str )
		{
			if ( strstr( $str, "'" ) !== false )
			{
				$output = '';
				$arr = explode( "'", $str );
				foreach( $arr as $item )
				{
					if ( strlen( $output ) > 0 )
					{
						$output .= ",\"'\",";
					}
					$output .= "'" . fileconv( $item ) . "'";	// 文字列がマルチバイトの場合、UTF-8に変換してやる必要がある。
				}
				$output = 'concat( ' . $output . ')';
			}
			else
			{
				$output = "'" . fileconv( $str ) . "'";	// 文字列がマルチバイトの場合、UTF-8に変換してやる必要がある。
			}
			
			return $output;
		}
		
		/* ---------------------------------------------------------------
		 * グループ条件をXPathの条件式として表現した文字列を返す。
		 * $groupKey : グループ条件。複数グループ条件を指定する場合、連想配列で指定する。
		 */
		public static function createXPathConditionForGroup( $groupKey )
		{
			$strCond = '';
			if ( is_array( $groupKey ) )
			{
				foreach ( $groupKey as $key => $value )
				{
					// valueをエスケープする。
					$escapedValue = cmsview_common::escapeStringInXPath( $value );
				
					// キー値単一指定の場合（旧仕様）の互換性維持。キー名に"cmsd_singlekey"が指定されていたら、キー名省略として単一キーで絞り込む。
					if ( $key == 'cmsd_singlekey' )
					{
						//$strCond = "groupkey='" . $value . "'";
						$strCond = "groupkey=" . $escapedValue;
						break;
					}
				
					// valueが空でない場合のみ実行。valueが空っぽの場合、条件が指定されなかったと見做して条件に追加しない。
					if ( strlen($value) > 0 ) // boolとして比較すると、例えば"0"が入っていた場合などにfalse扱いになってしまうのでNG。
					{
						if ( ! empty( $strCond ) )
						{
							$strCond .= " and ";
						}
						//$strCond .= "(groupkey/@key='" . $key . "' and groupkey='" . $value . "')";
						//$strCond .= "(groupkey[@key='" . $key . "']='" . $value . "')";
						$strCond .= "(groupkey[@key='" . $key . "']=" . $escapedValue . ")";
					}
				}
			}
			else
			{
				//$strCond = "groupkey='" . $groupKey . "'";
				$strCond = "groupkey=" . cmsview_common::escapeStringInXPath( $groupKey );
			}
			
			return $strCond;
		}
		
		
		/* ---------------------------------------------------------------
		 * 出力用DOM用のhref属性を生成する。
		 * $entryId : エントリID。
		 * $groupKey : グループ条件。複数グループ条件を指定する場合、連想配列で指定する。
		 */
		public static function createUrlParamsForEntry( $entryId, $groupKey )
		{
			$href = "?eid=" . $entryId;

			if ( is_array( $groupKey ) )
			{
				foreach ( $groupKey as $key => $value )
				{
					$href .= '&' . urlencode($key) . '=' . urlencode($value);
				}
			}
			else
			{
				// 何もしない。
				// グループキーがスカラーで渡されたということは、ページに埋め込みで直接指定されているということなので、
				// ここでURL渡しする必要はない。
			}
			
			return $href;
		}
		
		
		/* ---------------------------------------------------------------
		 * 出力用DOM用のrwhref属性を生成する。
		 * $entryId : エントリID。
		 * $groupKey : グループ条件。複数グループ条件を指定する場合、連想配列で指定する。
		 */
		public static function createRWUrlParamsForEntry( $entryId, $groupKey )
		{
			$rwhref = "eid/" . $entryId . ".html";

			return $rwhref;
		}
		
		
		/* ---------------------------------------------------------------
		 * 出力用DOMにグループ情報を追加する。
		 * $pageno : ページ番号
		 * $groupKey : グループ条件。複数グループ条件を指定する場合、連想配列で指定する。
		 */
		public static function createUrlParamsForPage( $pageno, $groupKey, $sortKey = null )
		{
			$href = "?pageno=" . $pageno;

			if ( is_array( $groupKey ) )
			{
				foreach ( $groupKey as $key => $value )
				{
					$href .= '&' . urlencode($key) . '=' . urlencode($value);
				}
			}
			else
			{
				// 何もしない。
				// グループキーがスカラーで渡されたということは、ページに埋め込みで直接指定されているということなので、
				// ここでURL渡しする必要はない。
			}
			
			if ( ! empty( $sortKey ) )
			{
				foreach ( $sortKey as $key => $value )
				{
					$href .= '&' . urlencode($key) . '=' . urlencode($value);
				}
			}
			
			return $href;
		}
		
		/* ---------------------------------------------------------------
		 * 出力用DOMにグループ情報を追加する。
		 * $groupKey : グループ条件。複数グループ条件を指定する場合、連想配列で指定する。
		 * &$xmlOutputRoot : 追加したい位置のDOM要素。
		 */
		public static function addGroupInfo( $groupKey, &$xmlOutputRoot )
		{
			// DOMドキュメントオブジェクトを取得
			$xmlDoc = $xmlOutputRoot->owner_document();
			
			// グループ情報を追加
			if ( is_array( $groupKey ) )
			{
				foreach ( $groupKey as $key => $value )
				{
					$xmlGroup = $xmlDoc->create_element('group');
					$xmlGroup->set_attribute('key', $key);
					$xmlGroup->set_attribute('value', fileconv( $value ) );
					$xmlOutputRoot->append_child($xmlGroup);
				}
			}
			else
			{
				$xmlGroup = $xmlDoc->create_element('group');
				$xmlGroup->set_attribute( 'value', fileconv( $groupKey ) );
				$xmlOutputRoot->append_child($xmlGroup);
			}
			
		}
		
		/* ---------------------------------------------------------------
		 * 出力用DOMにパラメータ情報を追加する。
		 * $paramKey : パラメータ名。複数パラメータを指定する場合、連想配列で指定する。
		 * &$xmlOutputRoot : 追加したい位置のDOM要素。
		 */
		public static function addParamInfo( $paramKey, &$xmlOutputRoot )
		{
			// DOMドキュメントオブジェクトを取得
			$xmlDoc = $xmlOutputRoot->owner_document();
			
			// パラメータ情報を追加
			foreach ( $paramKey as $key => $value )
			{
				$xmlParam = $xmlDoc->create_element('param');
				$xmlParam->set_attribute('key', $key);
				$xmlParam->set_attribute('value', fileconv( $value ) );
				$xmlOutputRoot->append_child($xmlParam);
			}
			
		}
		
		
		/* ---------------------------------------------------------------
		 * 出力用DOMにグループ情報を追加する。
		 * $groupKey : グループ条件。複数グループ条件を指定する場合、連想配列で指定する。
		 * &$xmlOutputRoot : 追加したい位置のDOM要素。
		 */
		public static function setGroupKeyValue( &$groupKeyValue )
		{
			// グループ条件がnullで指定されていた場合（ 'key'=>null ）、グループ指定をパラメータから読み込む。
			if ( is_array( $groupKeyValue ) )
			{
				foreach ( $groupKeyValue as $key=>$value )
				{
					// nullならば、リクエストの値を設定する。
					//if ( is_null($value) )
					if ( strlen( $value ) <= 0 ) //  2005.8.21 変更 : is_nullだと、コマンドタグから省略した場合に''が渡るのでNGだった。emptyだと0が指定された場合にまずいので、strlen()<=0を使用した。
					{
						// リクエストに指定があれば、それを設定する。
						if ( isset( $_REQUEST[$key] ) )
						{
							$groupKeyValue[$key] = $_REQUEST[$key];
						}
						else
						{
							// リクエストの指定がなければ、項目自体を削除する。
							unset( $groupKeyValue[$key] );
							
						}
					}
				}
			}
			
		}
		
		public static function IsEntryPublished( &$xmlEntryRoot, $publishControl )
		{
			// 年月日時分
			$sYear = sprintf( "%04d", date('Y') );
			$sMonth = sprintf( "%02d", date('m') );
			$sDay = sprintf( "%02d", date('d') );
			$sHour = sprintf( "%02d", date('H') );
			$sMinute = sprintf( "%02d", date('i') );

			//$strToday = $sYear . '-' . $sMonth . '-' . $sDay . ' ' . $sHour . ':' . $sMinute;
			$strToday = $sYear . $sMonth . $sDay . $sHour . $sMinute;
			
			// 「公開」かつ、開始日時≦現在日時≦終了日時 のものを抽出する条件式。
			// 公開フラグや開始日時、終了日時が存在しなかった頃のバージョンとの互換性の為に、それらが無いエントリも考慮した条件式にする。
			// ※XPathにて文字列の大小比較が出来ない事が判明。但し、数値文字列の場合のみ比較可能だったので、:や-などを外して比較することにした。
			
			$startDate = $xmlEntryRoot->get_attribute('startdate');
			$endDate = $xmlEntryRoot->get_attribute('enddate');
			$visible = $xmlEntryRoot->get_attribute('visible');
			
			// 公開方法によって条件式を変更
			
			$isPublished = false;
			switch ( $publishControl )
			{
				// 全て表示
				case 'none' :
					$isPublished = true;
					break;
				
				// 公開・非公開のみ
				case 'on-off' :
					$isPublished = ( empty( $visible) or $visible != 'False' );
					break;
				
				// タイマー公開も使う
				case 'full' :
					// まず、visibleが空ならば、古いエントリの為開始や終了も空ということになるので無条件に公開。
					// 次に、もしvisibleがFalseなら、無条件に非公開。開始日時＞現在日時（公開開始前）でも無条件に非公開。終了日時≦現在日時（公開終了後）でも、無条件に非公開。
					$isPublished = ( empty( $visible ) or  ! ( $visible == 'False' or $startDate > $strToday or $endDate <= $strToday ) );	// 対応するXPath式と同じ形式にした。
					break;
			}
			
			return $isPublished;
		}
		
	}
	
	
	/* ---------------------------------------------------------------
	 * エントリ１件用ビュークラス。
	 * スキーマ、エントリ、デザインを結びつけ、HTMLをレンダリングする。
	 */
	class entry_view
	{
		var $entryName = '';
		//var $designName = null;
		var $entryId = '';
		var $groupKeyValue = array();
		var $navigation = false;
		//var $xml = false;
		var $tagRemoving = false;
		
		var $paramKeyValue = array();	// 2005.9.25 導入. xsltに渡すデータに含めるパラメータを格納。
		var $interlock = false; // 2006.2.15 導入. これが true の時に eid の代わりに pageno を与える事で、eid を自動的に連動させる。
		var $pageMaxRows = 10;	// 2006.2.16 追加。連動対応。
		var $inlineXslt = null;  // 2007.7.3 導入. ここに文字列で直接xsltを与える事ができる。

		/* ---------------------------------------------------------------
		 * コンストラクタ。
		 * $entryName : エントリ名。
		 * $entryId : エントリID。（省略時はREQUEST"eid"から取得）
		 */
		function __construct( $entryName, $entryId = null )
		{
			$this->entryName = $entryName;
			$this->entryId = $entryId;
		}
		
		/* ---------------------------------------------------------------
		 * ユーザーパラメータの追加。
		 * $entryName : エントリ名。
		 * $keyValue : 省略時は値無し（URLから取得）。
		 */
		function addParamKey( $keyName, $keyValue = null )
		{
			if ( is_null( $this->paramKeyValue ) )
			{
				$this->paramKeyValue = array();
			}
			
			if ( is_null( $keyValue ) || $keyValue === '' )
			{
				$keyValue = getGETParam( $keyName, '' );
			}
			
			$this->paramKeyValue[ $keyName ] = $keyValue;
		}
		
		/* ---------------------------------------------------------------
		 * ユーザーパラメータの全消去。
		 */
		function clearParamKey()
		{
			$this->paramKeyValue = array();
		}
		
		/* ---------------------------------------------------------------
		 * 絞込み条件の追加。
		 * $keyName : キー名。
		 * $keyValue : キー値。（省略時はREQUESTから取得）
		 */
		function addGroupKey( $keyName, $keyValue = null )
		{
			if ( is_null( $this->groupKeyValue ) )
			{
				$this->groupKeyValue = array();
			}

			$this->groupKeyValue[ $keyName ] = $keyValue;
			
			$this->navigationOn();
		}
		
		/* ---------------------------------------------------------------
		 * 絞込み条件の全消去。
		 */
		function clearGroupKey()
		{
			$this->navigationOff();
			$this->groupKeyValue = array();
		}
		
		/* ---------------------------------------------------------------
		 * ナビゲーションをつける。
		 */
		function navigationOn()
		{
			$this->navigation = true;
		}
		
		/* ---------------------------------------------------------------
		 * ナビゲーションをつけない。
		 */
		function navigationOff()
		{
			$this->navigation = false;
		}
		
		/* ---------------------------------------------------------------
		 * 一覧詳細連動を行う。
		 */
		function interlockOn()
		{
			$this->interlock = true;
		}
		
		/* ---------------------------------------------------------------
		 * 一覧詳細連動を行わない。
		 */
		function interlockOff()
		{
			$this->interlock = false;
		}
		
		/* ---------------------------------------------------------------
		 * １ページ最大表示件数設定（連動用）。
		 */
		function setPageMaxRows( $value )
		{
			$this->pageMaxRows = $value;
		}
		
		/* ---------------------------------------------------------------
		 * １ページ最大表示件数取得（連動用）。
		 */
		function getPageMaxRows()
		{
			return $this->pageMaxRows;
		}
		
		/* ---------------------------------------------------------------
		 * タグ除去を行う。
		 */
		function tagRemovingOn()
		{
			$this->tagRemoving = true;
		}

		/* ---------------------------------------------------------------
		 * タグ除去を行わない（規定値）。
		 */
		function tagRemovingOff()
		{
			$this->tagRemoving = false;
		}
		
		/* ---------------------------------------------------------------
		 * インラインXSLTを指定
		 * $xsltString : XSL文字列を設定。xml宣言とxsl:stylesheet要素は自動的に補完される。
		 */
		function setInlineXslt( $xsltString )
		{
			$this->inlineXslt = $xsltString;
		}
		
		/* ---------------------------------------------------------------
		 * インラインXSLTを補完後の形で取得
		 */
		function getInlineXsltFull()
		{
			$xslt = '<?xml version="1.0" encoding="UTF-8" ?>' . "\n" .
					'<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">' . "\n" . 
					'<xsl:output method="xml" encoding="' . gCMSDInternalEncoding() . '" omit-xml-declaration="yes" indent="no" />' . "\n";
					
			$xslt .= $this->inlineXslt;
			
			$xslt .= '</xsl:stylesheet>';
			
			return $xslt;

		}

		/* ---------------------------------------------------------------
		 * XMLとしてエントリを出力する。
		 * $designName : デザイン名。省略した場合はXSL変換前のXMLを出力する。
		 */
		function outputxml( $designName = null )
		{
			print $this->output( $designName, true );
		}
		
		/* ---------------------------------------------------------------
		 * XHTMLとしてエントリを出力する。
		 * $designName : デザイン名。省略不可。
		 */
		function outputhtml( $designName )
		{
			print $this->output( $designName, false );
		}
		
		/* ---------------------------------------------------------------
		 * エントリの一部を直接出力する。
		 * $itemId : 項目名。省略不可。
		 */		
		function outputdata( $itemId )
		{
			print $this->output( $itemId, false, true );
		}
		
		
		/* ---------------------------------------------------------------
		 * XMLとしてエントリを出力する。
		 * $designName : デザイン名。省略した場合はXSL変換前のXMLを出力する。
		 */
		function getxml( $designName = null )
		{
			return $this->output( $designName, true );
		}
		
		/* ---------------------------------------------------------------
		 * XHTMLとしてエントリを出力する。
		 * $designName : デザイン名。省略不可。
		 */
		function gethtml( $designName )
		{
			return $this->output( $designName, false );
		}
		
		/* ---------------------------------------------------------------
		 * エントリの一部を直接出力する。
		 * $itemId : 項目名。省略不可。
		 */		
		function getdata( $itemId )
		{
			return $this->output( $itemId, false, true );
		}
		
		
		/* ---------------------------------------------------------------
		 * エントリ1件の表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。（省略時は"default"）
		 * $entryId : エントリID。（省略時はREQUEST"eid"から取得）
		 * $showNavi : ナビゲーションを出力するかどうか。
		 * $isXML : XSLTを適用せず、XMLのまま出力するかどうか( true : XMLのまま出力する。 )
		 */
		//function entry( $entryName, $designName = null, $entryId = '', $groupKeyValue = '', $showNavi = false, $isXML = false )		
		function output( $designName = null, $isXML = false, $onlyData = false )
		{
			// 変数初期化
			$entryName = $this->entryName;
			//$designName = $this->designName;
			$entryId = $this->entryId;
			$groupKeyValue = $this->groupKeyValue;
			$paramKeyValue = $this->paramKeyValue;
			//$isXML = $this->xml;
			$showNavi = $this->navigation;
			$tagRemoving = $this->tagRemoving;
			
			$interlock = $this->interlock;	// 2006.2.16 追加。連動対応。
			$pageNo = 0;	// 2006.2.16 追加。連動対応。
			$pageMaxRows = $this->pageMaxRows;	// 2006.2.16 追加。連動対応。
			
			// Configの読み込み
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $entryName );
			
			// グループ条件がnullで指定されていた場合（ 'key'=>null ）、グループ指定をパラメータから読み込む。
			cmsview_common::setGroupKeyValue( $groupKeyValue );
			
			// ページ番号の取得。一覧詳細連動用。2006.02.16 追加。
			if ( isset( $_REQUEST['pageno'] ) )
			{
				$pageNo = $_REQUEST['pageno'];
				
				// 取得したページ番号が数値以外なら、指定なしとする。
				if ( ! is_numeric( $pageNo ) )
				{
					$pageNo = 0;	// 0:指定なし
				}
			}
			
			
			// エントリIDが省略された場合、REQUESTから取得する。
			if ( empty( $entryId ) and isset( $_REQUEST[ 'eid' ] ) )
			{
				$entryId = $_REQUEST[ 'eid' ];
			}
			
			// REQUESTからも取得できない場合
			if ( empty( $entryId ) ) 
			{
				// 連動ONで、且つ、ページ番号が指定されていたら、"interlock"とする。
				if ( $interlock and $pageNo != 0 )
				{
					$entryId = 'interlock';
				}
				else
				{
					// 連動も有効でない場合、"top"が指定されたものとみなす。
					$entryId = 'top';
				}
			}
			
			//cmsd_abort_proc( "pageno='$pageNo', entryId='$entryId', interlock=" . ( $interlock == true ? 'True' : 'False' ) );
			
			// エントリIDに"top"が指定された場合、indexファイルの先頭エントリを適用する。
			if ( $entryId == 'top' or $entryId == 'interlock' or $entryId == 'random' )
			{
				// indexDOMを生成
		   	 	$xmlDocIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
				$xpathIndex = $xmlDocIndex->xpath_new_context();
				
				/*
				// 連動時のエントリ範囲（ページ）絞込み条件生成
				if ( $entryId == 'interlock' )
				{
					// ページ番号からエントリ範囲を決定
					$startPos = $pageMaxRows * ( $pageNo - 1) + 1;
					$endPos = $pageMaxRows * ( $pageNo );
					
					$rangecond = "[position()>=" . $startPos . " and position()<=" . $endPos . "]";
				}
				*/
				
				// グループによる絞込み条件生成
				$cond = cmsview_common::createXPathCondition( $groupKeyValue, $entryConfig->publishControl );
				
				// 連動時のエントリ位置条件生成。pageNo と pageMaxRows より、該当ページの先頭エントリを見つける。
				if ( $entryId == 'interlock' )
				{
					$interlockEntryPos = $pageMaxRows * ( $pageNo - 1 ) + 1;
					$cond .= "[$interlockEntryPos]";
				}
				
				// 絞込み
				$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" . $cond );
				
				//cmsd_abort_proc(	"/entrylist/entry" . $cond . '[' . $interlockEntryPos . ']' );
				
				// 結果が存在すれば、そのエントリのIDを取得する。
				if ( !empty($xmlAllEntryRefArr->nodeset) )
				{
					if ( $entryId == 'interlock' or $entryId == 'top' )
					{
						// interlock または top の場合、結果の先頭エントリIDを取得する。
						$entryId = $xmlAllEntryRefArr->nodeset[0]->get_attribute('id');
					}
					elseif ( $entryId == 'random' )
					{
						// random の場合、ランダムに一つ選び出す。
						$randomIndex = mt_rand( 0, count( $xmlAllEntryRefArr->nodeset ) - 1 );
						$entryId = $xmlAllEntryRefArr->nodeset[ $randomIndex ]->get_attribute('id');
					}
				}
				
				$xmlDocIndex->free();
			}
			
		
			// スキーマ名の取得
			$schemaName = cmsview_common::getSchemaName( $entryName );
			
			// スキーマ情報を配列形式で取得
			$schemaInfoArr = getSchemaInfoByArray( $schemaName );

			// 指定のデザインファイルが存在しない場合、エラーメッセージを表示して終了する。
			if ( ! $onlyData and ( ! $isXML || $designName ) )	// XML指定じゃない場合は無条件でチェック、designNameが指定されていれば無条件でチェック。onlyDataの場合はチェックしない。
			{
				// 但し、designNameが指定されていなくても、inlineXsltが指定されている場合は問題なし。
				if ( strlen( $designName ) > 0 )
				{
			
					if ( ! file_exists( App::createXslFilePath( $schemaName, $designName ) ))
					{
						return "ファイル'" . App::createXslFilePath( $schemaName, $designName ) . "'が見つかりません。又は埋め込みコマンドのデザイン名の指定が間違っています。";
					}
					
				}
				elseif ( strlen( $this->inlineXslt ) <= 0 )
				{
					return "埋め込みコマンドのdesign属性、又はcmsd:design要素がありません。";
				}
			
			

			}
			

			// 指定のエントリファイルが存在しない場合、空っぽのエントリデータを生成する。
			if ( ! file_exists( App::createEntryFilePath( $entryName, $entryId ) ) )
			{
				// 空っぽのエントリデータ「noentry」要素を生成する。
				
				// DOMをを生成。
				$xmlDocEntry = domxml_new_doc( '1.0' );
				
				// ルートノード'noentry'を生成。
				$xmlEntryRoot = $xmlDocEntry->add_root( 'noentry' );
			}
			else 
			{
				// エントリが存在するなら、該当エントリを読み込んで出力用に加工する。
							
				// エントリDOMを生成
		   	 	$xmlDocEntry = domxml_open_file( App::createEntryFilePath( $entryName, $entryId ) );
	
				// DOMツリーの内容のtext/textarea項目を、出力用に調整する。
				trans_dom_for_output( $xmlDocEntry, $schemaInfoArr, $tagRemoving );
				
				// DOMツリーの内容のメニュー項目を、出力用に調整する。2014/9/2 added.
				set_menuitem_info( $xmlDocEntry, $schemaInfoArr, $entryConfig );
	
				// エントリDOMのルート要素を取得
				$xmlEntryRoot = $xmlDocEntry->document_element();
				
				// 非公開且つ、コンテンツ管理画面にログインしていない状態の場合、noentryとする。
				if ( ! ( cmsview_common::IsEntryPublished( $xmlEntryRoot, $entryConfig->publishControl ) or ( LoginManager::IsAuthed() and cmsview_common::IsShowingClosedEntry() ) ) )
				//if ( ! cmsview_common::IsEntryPublished( $xmlEntryRoot ) and ! LoginManager::IsAuthed() and ! cmsview_common::IsShowingClosedEntry() )
				{
					// 空っぽのエントリデータ「noentry」要素を生成する。
					
					// DOMをを生成。
					$xmlDocEntry = domxml_new_doc( '1.0' );
					
					// ルートノード'noentry'を生成。
					$xmlEntryRoot = $xmlDocEntry->add_root( 'noentry' );
					
				}
				else
				{
					//print "IsAuthed = " . ( LoginManager::IsAuthed() ? "True" : "False" ) . ", IsEntryPublished = " . ( cmsview_common::IsEntryPublished( $xmlEntryRoot ) ? "True" : "False" ) . ", IsShowingClosedEntry = " . ( cmsview_common::IsShowingClosedEntry()	? "True" : "False" ) . "<br />";
					
					// entryタグにhref属性を追加する。
					$xmlEntryRoot->set_attribute('href', cmsview_common::createUrlParamsForEntry( $entryId, $groupKeyValue ) );
					
					// 【試験導入2005.05.12 for 1.0.0f】entryタグにrwhref属性を追加する。
					//$xmlEntryRoot->set_attribute('rwhref', cmsview_common::createRWUrlParamsForEntry( $entryId, $groupKeyValue ) );			
					
					// entryタグにdaysago属性を追加する。
					set_daysago_attribute( $xmlEntryRoot );
				
					// ナビゲーション出力フラグがTrueならば、ナビゲーションを出力する。
					if ( $showNavi )
					{
						// indexDOMを生成
				   	 	$xmlDocIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
						$xpathIndex = $xmlDocIndex->xpath_new_context();
						
						// グループによる絞込み
						$cond = cmsview_common::createXPathCondition( $groupKeyValue, $entryConfig->publishControl );
						$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" . $cond );
						
						/*
						// グループによる絞込み
						if ( empty($groupKeyValue))
						{
							// グループキーが指定されていなければ、全て取得する。
							$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
						}
						else
						{
							// グループキーが指定されていたら、グループで絞り込む。
							$cond = cmsview_common::createXPathConditionForGroup( $groupKeyValue );
							if ( strlen( $cond ) > 0 )
							{
								$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[$cond]" );
							}
							else
							{
								$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
							}
						}
						*/
					
						// グループ内全件数
						$allEntryCount = count( cmsd_get_nodeset( $xmlAllEntryRefArr ) );
					
						// カレント位置
						$entryPos = -1;	// 初期値
						foreach ( cmsd_get_nodeset( $xmlAllEntryRefArr ) as $currEntryPos => $xmlEntryRef )
						{
							$currEntryId = $xmlEntryRef->get_attribute('id');
							//print $currEntryId . "=$entryId, ";
							if ( $currEntryId == $entryId )
							{
								$entryPos = $currEntryPos;
								break;
							}
						}
						
				
						if ( $entryPos >= 0 )
						{
							// navi要素を生成
							$xmlNavi = $xmlDocEntry->create_element('navi');
							$xmlNavi->set_attribute('position', $entryPos );
							$xmlNavi->set_attribute('allcount', $allEntryCount );
							
							// 前エントリが存在すれば、prev要素を追加
							if ( $entryPos > 0	)
							{
								$xmlPrev = $xmlDocEntry->create_element('prev');
								$id = $xmlAllEntryRefArr->nodeset[$entryPos-1]->get_attribute('id');
								$xmlPrev->set_attribute('id', $id );
								$xmlPrev->set_attribute('href', cmsview_common::createUrlParamsForEntry( $id, $groupKeyValue ) );
								// 【試験導入2005.05.12 for 1.0.0f】rwhref属性を追加する。
								//$xmlPrev->set_attribute('rwhref', cmsview_common::createRWUrlParamsForEntry( $id, $groupKeyValue ) );
								$xmlNavi->append_child( $xmlPrev );
							}
						
							// 次ページが存在すれば、next要素を追加
							if ( $allEntryCount > $entryPos + 1 )
							{
								$xmlNext = $xmlDocEntry->create_element('next');
								$id = $xmlAllEntryRefArr->nodeset[$entryPos+1]->get_attribute('id');
								$xmlNext->set_attribute('id', $id );
								$xmlNext->set_attribute('href', cmsview_common::createUrlParamsForEntry( $id, $groupKeyValue ) );
								// 【試験導入2005.05.12 for 1.0.0f】rwhref属性を追加する。
								//$xmlNext->set_attribute('rwhref', cmsview_common::createRWUrlParamsForEntry( $id, $groupKeyValue ) );
								$xmlNavi->append_child( $xmlNext );
							}
						
							$xmlEntryRoot->append_child( $xmlNavi );
						}
						
						$xmlDocIndex->free();
						
					}
				}
			}
			
			// データ項目のみの出力フラグがONならば、designNameをデータ項目名とみなして項目のみを出力して終了する。
			if ( $onlyData )
			{
				$xpathEntry = $xmlDocEntry->xpath_new_context();
				$xmlDataArr = $xpathEntry->xpath_eval( "/entry/$designName" );
				if ( count( cmsd_get_nodeset( $xmlDataArr ) ) > 0 )
				{
					return convencoding_xml2html( $xmlDataArr->nodeset[0]->get_content() );
				}
				return;
			}
			
			// グループが指定されていた場合、グループ情報を出力する。
			if ( ! empty( $groupKeyValue ) )
			{
				cmsview_common::addGroupInfo( $groupKeyValue, $xmlEntryRoot );
			}
			
			// パラメータが指定されていた場合、パラメータ情報を出力する。
			if ( ! empty( $paramKeyValue ) )
			{
				cmsview_common::addParamInfo( $paramKeyValue, $xmlEntryRoot );
			}
						
			// today要素を追加
			set_today_info( $xmlEntryRoot );
			
			
			//print $xmlDocEntry->dump_mem( true, 'EUC-JP');
			
			/*
			$xslt = domxml_xslt_stylesheet_file( App::createXslFilePath( $schemaName, $designName) );
			$result = $xslt->process( $xmlDocEntry );
			print $result->dump_mem( true, 'EUC-JP' );
			*/
			
			if ( $isXML )
			{
				// デザイン定義名nullの場合はXSL変換なし。
				if ( $designName )
				{
					// XML出力はUTF-8固定
					$result = ADXslt::process( $xmlDocEntry, App::createXslFilePath( $schemaName, $designName), 'UTF-8' );
				}
				elseif ( $this->inlineXslt != null )
				{
					$result = ADXslt::process( $xmlDocEntry, $this->getInlineXsltFull(), 'UTF-8', true );	// 最後の true は、xsltが文字列として渡される事を意味する。
				}
				else
				{
					// XML出力はUTF-8固定
					$result = $xmlDocEntry->dump_mem( true, 'UTF-8' );
				}
			}
			else
			{
				if ( $designName )
				{
					$result = ADXslt::process( $xmlDocEntry, App::createXslFilePath( $schemaName, $designName) );
				}
				else
				{
					$result = ADXslt::process( $xmlDocEntry, $this->getInlineXsltFull(), gCMSDInternalEncoding(), true );	// 最後の true は、xsltが文字列として渡される事を意味する。
				}
			}
			
			$xmlDocEntry->free();
			
			return $result;			
		}
	}
	

	
	/* ---------------------------------------------------------------
	 * エントリ一覧用ビュークラス。
	 * スキーマ、エントリ、デザインを結びつけ、HTMLをレンダリングする。
	 */
	class entrylist_view
	{
		var $entryName = '';
		//var $designName = null;
		var $pageNo = '';
		var $groupKeyValue = array();
		var $navigation = true;
		//var $xml = false;
		var $tagRemoving = false;
		var $sorting = false;
		var $sortKeyValue = array();
		var $cols = 1;
		
		var $paramKeyValue = array();	// 2005.9.25 導入. xsltに渡すデータに含めるパラメータを格納。
		var $entryId = '';	// 2005.9.29 導入. 現在表示されている（かもしれない）entryIdを格納。
		
		var $interlock = false; // 2006.2.15 導入. これが true の時に pageno の代わりに eid を与える事で、pageno を自動的に連動させる。
		
		var $inlineXslt = null;  // 2007.7.3 導入. ここに文字列で直接xsltを与える事ができる。
		
		/* ---------------------------------------------------------------
		 * コンストラクタ。
		 * $entryName : エントリ名。
		 * $entryId : エントリID。（省略時はREQUEST"eid"から取得）
		 */
		function __construct( $entryName, $pageNo = null )
		{
			$this->entryName = $entryName;
			
			// 'top'指定の場合、1が指定されたとみなす。
			if ( $pageNo == 'top' )
			{
				$pageNo = 1;
				//$this->pageNo = 1;
			}
			
			if ( empty( $pageNo ) )
			{
				// ページ番号省略の場合、URL引数による動的切り替えになるので、自動的にナビゲーションをOnにする。
				$this->navigationOn();
				$this->pageNo = null;
			}
			else
			{
				if ( is_numeric( $pageNo ) )
				{
					$this->navigationOff();		// ページ番号直接指定の場合、自動的にナビゲーションをOffにする。
					$this->pageNo = $pageNo;
				}
				else
				{
					cmsd_abort_proc( "ページ番号に不正な値('$pageNo')が指定されました。整数値か'top'を指定してください。" );
				}
			}
			
		}
		
		/* ---------------------------------------------------------------
		 * エントリIDを指定
		 * $entryId : エントリID。※実際にコード上で指定する事は稀で、普通はURLパラメータから取得される。
		 */
		function setEntryId( $entryId )
		{
			$this->entryId = $entryId;
		}
		
		/* ---------------------------------------------------------------
		 * インラインXSLTを指定
		 * $xsltString : XSL文字列を設定。xml宣言とxsl:stylesheet要素は自動的に補完される。
		 */
		function setInlineXslt( $xsltString )
		{
			$this->inlineXslt = $xsltString;
		}
		
		/* ---------------------------------------------------------------
		 * インラインXSLTを補完後の形で取得
		 */
		function getInlineXsltFull()
		{
			$xslt = '<?xml version="1.0" encoding="UTF-8" ?>' . "\n" .
					'<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">' . "\n" . 
					'<xsl:output method="xml" encoding="' . gCMSDInternalEncoding() . '" omit-xml-declaration="yes" indent="no" />' . "\n";
					
			$xslt .= $this->inlineXslt;
			
			$xslt .= '</xsl:stylesheet>';
			
			return $xslt;

		}
		
		/* ---------------------------------------------------------------
		 * グリッド形式の出力をしたい場合に設定する「列数」を指定。
		 * $cols : 2以上を指定。1を指定した場合は通常のエントリ一覧出力になる。
		 */
		function setCols( $cols )
		{
			$this->cols = $cols;
		}
		
		/* ---------------------------------------------------------------
		 * ユーザーパラメータの追加。
		 * $entryName : エントリ名。
		 * $keyValue : 省略時は値無し（URLから取得）。
		 */
		function addParamKey( $keyName, $keyValue = null )
		{
			if ( is_null( $this->paramKeyValue ) )
			{
				$this->paramKeyValue = array();
			}
			
			if ( is_null( $keyValue ) || $keyValue === '' )
			{
				$keyValue = getGETParam( $keyName, '' );
			}
			
			$this->paramKeyValue[ $keyName ] = $keyValue;
		}
		
		/* ---------------------------------------------------------------
		 * ユーザーパラメータの全消去。
		 */
		function clearParamKey()
		{
			$this->paramKeyValue = array();
		}		
		
		/* ---------------------------------------------------------------
		 * 絞込み条件の追加。
		 * $entryName : エントリ名。
		 * $keyValue : 省略時は値無し（URLから取得）。
		 */
		function addGroupKey( $keyName, $keyValue = null )
		{
			if ( is_null( $this->groupKeyValue ) )
			{
				$this->groupKeyValue = array();
			}

			$this->groupKeyValue[ $keyName ] = $keyValue;
		}
		
		/* ---------------------------------------------------------------
		 * 絞込み条件の全消去。
		 */
		function clearGroupKey()
		{
			$this->groupKeyValue = array();
		}
		
		/* ---------------------------------------------------------------
		 * ナビゲーションをつける。
		 */
		function navigationOn()
		{
			$this->navigation = true;
		}
		
		/* ---------------------------------------------------------------
		 * ナビゲーションをつけない。
		 */
		function navigationOff()
		{
			$this->navigation = false;
		}
		
		function interlockOn()
		{
			$this->interlock = true;
		}
		
		function interlockOff()
		{
			$this->interlock = false;
		}
		
		/* ---------------------------------------------------------------
		 * タグ除去を行う。
		 */
		function tagRemovingOn()
		{
			$this->tagRemoving = true;
		}

		/* ---------------------------------------------------------------
		 * タグ除去を行わない（規定値）。
		 */
		function tagRemovingOff()
		{
			$this->tagRemoving = false;
		}
		
		/* ---------------------------------------------------------------
		 * タグ除去を行う。
		 */
		function sortingOn()
		{
			$this->sorting = true;
		}

		/* ---------------------------------------------------------------
		 * タグ除去を行わない（規定値）。
		 */
		function sortingOff()
		{
			$this->sorting = false;
		}
		
		/* ---------------------------------------------------------------
		 * ソート条件の追加。
		 * $entryName : エントリ名。
		 * $entryId : エントリID。（省略時はREQUEST"eid"から取得）
		 */
		function addSortKey( $sortId, $sortParams = null )
		{
			if ( is_null( $this->sortKeyValue ) )
			{
				$this->sortKeyValue = array();
			}

			$this->sortKeyValue[ $sortId ] = $sortParams;
			
			$this->sortingOn();
		}
		
		/* ---------------------------------------------------------------
		 *	ソート条件の全消去。
		 */
		function clearSortKey()
		{
			$this->sortingOff();
			
			$this->sortKeyValue = array();
			
		}		
				
		/* ---------------------------------------------------------------
		 * XMLとしてエントリを出力する。
		 * $designName : デザイン名。省略した場合はXSL変換前のXMLを出力する。
		 * $pageMaxRows : 1ページ当たりの行数。（省略値=10）* $pageMaxRows : 1
		 */
		function outputxml( $designName = null, $pageMaxRows = 10 )
		{
			print $this->outputcommon( $designName, $pageMaxRows, true );
		}
		
		/* ---------------------------------------------------------------
		 * XHTMLとしてエントリを出力する。
		 * $designName : デザイン名。省略不可。
		 * $pageMaxRows : 1ページ当たりの行数。（省略値=10）
		 */
		function outputhtml( $designName, $pageMaxRows = 10 )
		{
			print $this->outputcommon( $designName, $pageMaxRows, false );
		}
		
		
		/* ---------------------------------------------------------------
		 * XMLとしてエントリを出力する。ナビゲーションはつけない。
		 * $designName : RSS変換用のデザイン名。
		 */
		function outputrss( $designName )
		{
			$this->navigationOff();
			print $this->outputcommon( $designName, 15, true );
		}
		
		
		/* ---------------------------------------------------------------
		 * XMLとしてエントリを出力する。
		 * $designName : デザイン名。省略した場合はXSL変換前のXMLを出力する。
		 * $pageMaxRows : 1ページ当たりの行数。（省略値=10）* $pageMaxRows : 1
		 */
		function getxml( $designName = null, $pageMaxRows = 10 )
		{
			return $this->outputcommon( $designName, $pageMaxRows, true );
		}
		
		/* ---------------------------------------------------------------
		 * XHTMLとしてエントリを出力する。
		 * $designName : デザイン名。省略不可。
		 * $pageMaxRows : 1ページ当たりの行数。（省略値=10）
		 */
		function gethtml( $designName, $pageMaxRows = 10 )
		{
			return $this->outputcommon( $designName, $pageMaxRows, false );
		}
		
		
		/* ---------------------------------------------------------------
		 * XMLとしてエントリを出力する。ナビゲーションはつけない。
		 * $designName : RSS変換用のデザイン名。
		 */
		function getrss( $designName )
		{
			$this->navigationOff();
			return $this->outputcommon( $designName, 15, true );
		}

		
		
		/* ---------------------------------------------------------------
		 * エントリリストの表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。
		 * $pageMaxRows : 1ページ当たりの行数。（省略値=10）
		 * $groupKeyValue : グループキー。（省略値=キーなし）
		 * $showNavi : ナビゲーションを出力するかどうか。
		 * $pageNo : ページ番号。（省略時はGETパラメータ"pageno"から取得）
		 * $isRSS : RSSかどうか。（省略時はfalse）
		 * $isXML : XMLとして出力するか。 (省略時はfalse)
		 */
		//function listpage( $entryName, $designName = null, $pageMaxRows = 10, $groupKeyValue = '', $showNavi = true, $pageNo = NULL, $isRSS = false, $isXML = false )
		function outputcommon( $designName = null, $pageMaxRows = 10, $isXML = false )
		{		
			$entryName = $this->entryName;
			$pageNo = $this->pageNo;
			$groupKeyValue = $this->groupKeyValue;
			$paramKeyValue = $this->paramKeyValue;
			$showNavi = $this->navigation;
			$tagRemoving = $this->tagRemoving;
			
			$sorting = $this->sorting;
			$sortKeyValue = $this->sortKeyValue;
			
			$cols = $this->cols;
			$entryId = $this->entryId;
			$interlock = $this->interlock;	// 2006.2.15 追加。連動対応。
			
			
			// Configの読み込み
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $entryName );
			
			// エントリIDが指定されていない場合、エントリIDをURLパラメータから取得を試みる
			// ※このエントリIDは、「エントリ一覧と共に、現在表示されている（かもしれない）エントリID」である。「連動」指定だった場合にエントリIDからpagenoを割り出す為に使用する。
			if ( empty($entryId) and isset( $_REQUEST['eid'] ) )
			{
					$entryId = $_REQUEST['eid'];
			}
			
			// ページ番号の取得
			if ( empty($pageNo) )	// pagenoが直接指定されていなければ
			{
				if ( isset( $_REQUEST['pageno'] ) )		// リクエストをチェックし、指定されていればpagenoを読み込む。
				{
					$pageNo = $_REQUEST['pageno'];
				}
			}
			
			if ( empty($pageNo) or !is_numeric($pageNo) )	// まだpagenoが取得できていないか、取得できていても値が異常だったら
			{
				if ( $interlock and ! empty($entryId) )	// 連動指定で且つ、エントリIDが指定されているか？
				{
					$pageNo = 0;	// 0:連動（後で抽出する）
				}
				else
				{
					$pageNo = 1;	// 最初のページ番号を設定する。
				}
			}
			
			// スキーマ名の取得
			$schemaName = cmsview_common::getSchemaName( $entryName );

			// スキーマ情報を配列形式で取得
			$schemaInfoArr = getSchemaInfoByArray( $schemaName );

			// 指定のデザインファイルが存在しない場合、エラーメッセージを表示して終了する。
			if ( ! $isXML || $designName )	// XML指定じゃない場合は無条件でチェック、designNameが指定されていれば無条件でチェック。
			{
				// 但し、designNameが指定されていなくても、inlineXsltが指定されている場合は問題なし。
				if ( strlen( $designName ) > 0 )
				{
			
					if ( ! file_exists( App::createListXslFilePath( $schemaName, $designName ) ))
					{
						return "ファイル'" . App::createListXslFileName( $schemaName, $designName ) . "'が見つかりません。又は埋め込みコマンドのdesign属性の指定が間違っています。";
						return;
					}
				
				}
				elseif ( strlen( $this->inlineXslt ) <= 0 )
				{
					return "埋め込みコマンドのdesign属性、又はcmsd:design要素がありません。";
				}
				
			}
			
			// indexDOMを生成
			if ( file_exists( App::createEntryFilePath( $entryName, 'index' ) ) )
			{
				// ファイルを読み込む。
		   	 	$xmlDocIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
			}
			else
			{
				// indexファイルが無い場合、ここで空のindexDOMを生成する。
				$xmlDocIndex = domxml_new_doc( '1.0' );
			
				// ルートノード'entry'を生成。
				$xmlDocIndex->add_root( 'entrylist' );
			}
			
			// リアルタイムソート：xmlDocIndexを、動的に生成するソート用XSLTを使ってソートする。
			if ( $sorting )
			{
				// ソート条件からXSLTを組み立てる。
				$sortTags = '';
				
				foreach ( $sortKeyValue as $id => $params )
				{
					$sortTag = '<xsl:sort ';
					if ( empty( $params ) )
					{
						$params = '';
						if ( isset( $_REQUEST[ $id ] ) )
						{
							$params = $_REQUEST[ $id ];
						}
						$sortKeyValue[$id] = $params;
					}
					
					// パラメータ内容のチェック
					if ( ! preg_match( '/^[,@a-zA-Z0-9_-]*$/i', $params ) )	// 英数字、及びカンマと@から構成される文字列のみ許可する。2007.06.15 eregを使うとアクセス集中時に固まる(?)不具合を回避。
					//if ( ! eregi( '^[,@a-zA-Z0-9_-]*$', $params ) )	// 英数字、及びカンマと@から構成される文字列のみ許可する。
					{
						print "ソートパラメータに不正な文字が使用されました。'$params'<br />";
						continue;
					}
					
					$select = '';
					$order = '';
					$dataType = '';
					$caseOrder = '';
					$lang = '';
					
					$paramsArr = explode( ",", $params );
					if ( isset( $paramsArr[0] ) ) { $select = $paramsArr[0]; }
					if ( isset( $paramsArr[1] ) ) { $order = $paramsArr[1]; }
					if ( isset( $paramsArr[2] ) ) { $dataType = $paramsArr[2]; }
					if ( isset( $paramsArr[3] ) ) { $caseOrder = $paramsArr[3]; }
					if ( isset( $paramsArr[4] ) ) { $lang = $paramsArr[4]; }
					
					//list( $select, $order, $dataType, $caseOrder, $lang ) = explode( ",", $params );
					
					if ( ! empty($select) )
					{
						if ( strpos( $select, '@' ) === false )	 // '==='で比較すると、戻り値の型も含めて比較する。単に==で比較すると、戻り値が0だった場合にfalseと認識されてしまう。
						{
							// @が入っていないということは、要素名が指定されたということ。index中ではgroupkey要素として保存されているので適切なpathに変換する。
							$sortTag .= 'select="groupkey[@key=\'' . $select . '\']" ';
						}
						else
						{
							// @が入っているということは、属性名が指定されたということ。
							$sortTag .= 'select="' . $select . '" ';
						}
					}
					
					if ( ! empty( $order ) )
					{
						switch ( $order )
						{
							case 'asc' :
								$order = 'ascending';
								break;
							
							case 'desc' :
								$order = 'descending';
								break;
							
							case 'ascending' :
							case 'descending' :
								break;
							
							default : 
								print "ソートパラメータのorderに不正が値が指定されました。'$order'<br />";
								continue;
						}
						
						$sortTag .= 'order="'. $order . '" ';
					}
					
					if ( ! empty( $dataType ) )
					{
						switch ( $dataType )
						{
							case 'text' :
							case 'number' :
								break;
							
							default : 
								print "ソートパラメータのdata-typeに不正が値が指定されました。'$dataTyper'<br />";
								continue;
						}
						
						$sortTag .= 'data-type="'. $dataType . '" ';
					}
					
					if ( ! empty( $caseOrder ) )
					{
						switch ( $caseOrder )
						{
							case 'upper-first' :
							case 'lower-first' :
								break;
							
							default : 
								print "ソートパラメータのcase-orderに不正が値が指定されました。'$caseOrder'<br />";
								continue;
						}
						
						$sortTag .= 'case-order="'. $caseOrder . '" ';
					}
					
					if ( ! empty( $lang ) )
					{
						$sortTag .= 'lang="'. $lang . '" ';
					}
					
					$sortTag .= '/>';
					
					// 最低でもソートキーが指定されていない場合は、ソート条件には加えない。
					if ( ! empty( $select ) )
					{
						$sortTags .= $sortTag;
					}
					
				}
				
				// ソート用XSL生成
				$strXslSort = <<<EOD
<?xml version="1.0" encoding="EUC-JP"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:template match="/">

<entrylist>

<xsl:for-each select="entrylist/entry">
$sortTags
<xsl:copy-of select="." />
</xsl:for-each>

</entrylist>

</xsl:template>
</xsl:stylesheet>
EOD;
				
				// ソート処理
				if ( ! empty( $sortTags ) )	// ソート条件が１つもない場合は行わない。
				{
					$strXmlDocIndexSorted = ADXslt::processStrXsl( $xmlDocIndex, $strXslSort );
					//$xmlDocIndex = null;
					$xmlDocIndex->free();
					$xmlDocIndex = domxml_open_mem( $strXmlDocIndexSorted );
				}

			}
			
			
			$xpathIndex = $xmlDocIndex->xpath_new_context();
			$xmlIndexRoot = $xmlDocIndex->document_element();
			
			// グループ条件がnullで指定されていた場合（ 'key'=>null ）、グループ指定をパラメータから読み込む。
			cmsview_common::setGroupKeyValue( $groupKeyValue );
			
			// グループ絞込み用のXPath部分文字列生成
			$cond = cmsview_common::createXPathCondition( $groupKeyValue, $entryConfig->publishControl );
			
			// ページ－エントリ連動処理 2006.2.16 追加
			if ( $pageNo == 0 )	//	0 : 連動
			{
				$xmlEntryIdArr = $xpathIndex->xpath_eval( "/entrylist/entry" . $cond . '/@id' );
				
				if ( !empty($xmlEntryIdArr->nodeset) )
				{
					foreach ( cmsd_get_nodeset( $xmlEntryIdArr ) as $entryIndex=>$xmlEntryId )
					{
						if ( $xmlEntryId->value == $entryId )	// ヒット！
						{
							// エントリ位置より、そのエントリが含まれるページ番号を計算してループを抜ける。
							$pageNo = floor( $entryIndex / $pageMaxRows ) + 1;
							break;
						}
					}
				}
				
				$xmlEntryIdArr = null;	// この結果は不要なのでここで開放。
				
				// ページ番号が決定しなかった場合は、規定値の1とする。
				if ( $pageNo == 0 )
				{
					$pageNo = 1;
				}
			}
			
			
			// ページ番号からエントリ範囲を決定
			$startPos = $pageMaxRows * ( $pageNo - 1) + 1;
			$endPos = $pageMaxRows * ( $pageNo );
			
			// グループとエントリ範囲による絞込み
			$xmlEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" . $cond . "[position()>=" . $startPos . " and position()<=" . $endPos . "]");
			
			// 出力用DOMを生成
			//$xmlDocOutput = domxml_new_doc( '1.0' );
			//$xmlOutputRoot = $xmlDocOutput->add_root( 'entrylist' );
			
			$strXmlOutput = '<?xml version="1.0" encoding="UTF-8"?>'. "\n";
			$strXmlOutput .= "<entrylist>\n";
			
			foreach( cmsd_get_nodeset( $xmlEntryRefArr ) as $entryIndex=>$xmlEntryRef )
			{
				// colsが2以上ならば、rowタグでエントリ群を囲む。
				if ( $cols >= 2 and  ( $entryIndex % $cols ) == 0 )
				{
					$strXmlOutput .= "<row>\n";
				}
				
				// エントリDOMを取得する。
				$href = $xmlEntryRef->get_attribute('href');
				$xmlDocEntry = domxml_open_file( App::createEntryDirPath( $entryName ) . $href );
				$xmlEntryRoot = $xmlDocEntry->document_element();
				
				// エントリにhref属性を追加する。
				$xmlEntryRoot->set_attribute('href', cmsview_common::createUrlParamsForEntry( $xmlEntryRoot->get_attribute('id'), $groupKeyValue ));
				
				// エントリにdaysago属性を追加する。
				set_daysago_attribute( $xmlEntryRoot );
				
				// エントリが「現在表示されている（かもしれない）エントリ」のエントリIDと同一ならば、印をつける。
				if ( $xmlEntryRoot->get_attribute('id') == $entryId )
				{
					$xmlEntryRoot->set_attribute('current', 'True');	// この'True'という値には意味はない。カレントでなければこの属性自体が存在しないので、XSL側では<xsl:if test="@current">と書けば良い。
				}
				
				// エントリIDが特に指定されていなかった場合、ページ最初のエントリをカレントとする。
				if ( strlen( $entryId ) <= 0 and $entryIndex == 0 )
				{
					$xmlEntryRoot->set_attribute('current', 'True');	// この'True'という値には意味はない。カレントでなければこの属性自体が存在しないので、XSL側では<xsl:if test="@current">と書けば良い。
				}
				
				// エントリをコピーして、出力用DOMにappendする。
				//$xmlOutputEntry = $xmlEntryRoot->clone_node();
				//$xmlOutputRoot->append_child( $xmlOutputEntry );
				
				$strXmlOutput .= $xmlDocEntry->dump_node( $xmlEntryRoot );
				
				// rowタグを閉じる。
				if ( $cols >= 2 and  ( $entryIndex % $cols ) == ( $cols - 1 ) )
				{
					$strXmlOutput .= "</row>\n";
				}
				
				//unset( $xmlEntryRoot );
				//unset( $xmlDocEntry );
				
				$xmlDocEntry->free();
				
			}
			
			// rowタグの閉じ漏れチェック
			if ( count( cmsd_get_nodeset( $xmlEntryRefArr ) ) > 0 and $cols >= 2 and  ( $entryIndex % $cols ) != ( $cols - 1 ) )
			{
				$strXmlOutput .= "</row>\n";
			}
			
			$strXmlOutput .= "</entrylist>";
			
			// 出力用XMLをDOM化
			$xmlDocOutput = domxml_open_mem( $strXmlOutput );

			//print '<pre>' . htmlentities( mb_convert_encoding( $xmlDocOutput->dump_mem(true, 'UTF-8'), 'EUC-JP' , 'UTF-8' ), ENT_COMPAT, 'EUC-JP' )	. '</pre>';
						
			// 出力用DOMのテキスト項目を出力用に変換
			trans_dom_for_output( $xmlDocOutput, $schemaInfoArr, $tagRemoving );
			
			// DOMツリーの内容のメニュー項目を、出力用に調整する。2014/9/2 added.
			set_menuitem_info( $xmlDocOutput, $schemaInfoArr, $entryConfig );
			
			
			// ナビゲーションの追加
			if ( $showNavi )
			{
				// グループによる絞込み
				$cond = cmsview_common::createXPathCondition( $groupKeyValue, $entryConfig->publishControl );
				$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" . $cond );
				
				/*
				if ( empty($groupKeyValue))
				{
					// グループキーが指定されていなければ、全て取得する。
					$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
				}
				else
				{
					// グループキーが指定されていたら、グループで絞り込む。
   					$cond = cmsview_common::createXPathConditionForGroup( $groupKeyValue );
					if ( strlen( $cond ) > 0 )
					{
						$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[" . $cond . "]" );
					}
					else
					{
						$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
					}
				}
				*/
				
				// ページ内のエントリ数
				$pageEntryCount = count( cmsd_get_nodeset( $xmlEntryRefArr ) );
				// グループ内の全エントリ数
				$allEntryCount = count( cmsd_get_nodeset( $xmlAllEntryRefArr ) );
				// 最大ページ数
				$maxPage = (int)( (int)$allEntryCount / (int)$pageMaxRows ) + (( $allEntryCount % $pageMaxRows > 0 ) ? 1 : 0 );
				
				$xmlOutputRoot = $xmlDocOutput->document_element();
				
				// navi要素を生成
				$xmlNavi = $xmlDocOutput->create_element('navi');
				$xmlNavi->set_attribute('position', $pageNo );
				$xmlNavi->set_attribute('allcount', $allEntryCount );
				$xmlNavi->set_attribute('maxpage', $maxPage );
				$xmlNavi->set_attribute('maxpagerows', $pageMaxRows );
				
				// 前ページが存在すれば、prev要素を追加
				if ( $pageNo > 1  )
				{
					$xmlPrev = $xmlDocOutput->create_element('prev');
					$xmlPrev->set_attribute('id', $pageNo - 1 );
					$xmlPrev->set_attribute('href', cmsview_common::createUrlParamsForPage( $pageNo - 1, $groupKeyValue, $sortKeyValue ) );
					$xmlNavi->append_child( $xmlPrev );
				}
				
				// 存在するページ数分、page要素を追加
				for ( $index = 1; $index <= $maxPage; $index++ )
				{
					$xmlPage = $xmlDocOutput->create_element('page');
					$xmlPage->set_attribute('id', $index );
					$xmlPage->set_attribute('href', cmsview_common::createUrlParamsForPage( $index, $groupKeyValue, $sortKeyValue ) );
					$xmlNavi->append_child( $xmlPage );
				}
				
				// 次ページが存在すれば、next要素を追加
				if ( $allEntryCount > $pageMaxRows * $pageNo )
				{
					$xmlNext = $xmlDocOutput->create_element('next');
					$xmlNext->set_attribute('id', $pageNo + 1 );
					$xmlNext->set_attribute('href', cmsview_common::createUrlParamsForPage( $pageNo + 1, $groupKeyValue, $sortKeyValue ) );
					$xmlNavi->append_child( $xmlNext );
				}
				
				$xmlOutputRoot->append_child( $xmlNavi );
				
			}
			
			// 出力DOMのドキュメントルート要素を取得
			$xmlOutputRoot = $xmlDocOutput->document_element();
			
			// today要素を追加
			set_today_info( $xmlOutputRoot );
			
			// グループが指定されていた場合、グループ情報を出力する。
			if ( ! empty( $groupKeyValue ) )
			{
				cmsview_common::addGroupInfo( $groupKeyValue, $xmlOutputRoot );
			}
			
			// パラメータが指定されていた場合、パラメータ情報を出力する。
			if ( ! empty( $paramKeyValue ) )
			{
				cmsview_common::addParamInfo( $paramKeyValue, $xmlOutputRoot );
			}
			
			// 現在表示されている（かもしれない）エントリID情報を出力する。
			if ( ! empty( $entryId ) )
			{
				$xmlOutputRoot->set_attribute('currenteid', $entryId );
			}
			
			// XSL変換処理
			if ( $isXML )
			{
				// デザイン定義名nullの場合はXSL変換なし。
				if ( $designName )
				{
					// XML出力はUTF-8固定
					$result = ADXslt::process( $xmlDocOutput, App::createListXslFilePath( $schemaName, $designName), 'UTF-8' );
				}
				elseif ( $this->inlineXslt != null )
				{
					// XML出力はUTF-8固定
					$result = ADXslt::process( $xmlDocOutput, $this->getInlineXsltFull(), 'UTF-8', true );	// 最後の true は、xsltが文字列として渡される事を意味する。
				}
				else
				{
					// XML出力はUTF-8固定
					$result = $xmlDocOutput->dump_mem( true, 'UTF-8' );	
				}
			}
			// 通常のページ表示変換処理
			else
			{
				if ( $designName )
				{
					$result = ADXslt::process( $xmlDocOutput, App::createListXslFilePath( $schemaName, $designName) );
				}
				else
				{
					$result = ADXslt::process( $xmlDocOutput, $this->getInlineXsltFull(), gCMSDInternalEncoding(), true );	// 最後の true は、xsltが文字列として渡される事を意味する。
				}
							
				
			}
			
			$xmlDocOutput->free();
			return $result;
			
		}
		
	}		



	/* ---------------------------------------------------------------
	 * 検索結果用ビュークラス。
	 * 指定されたエントリフォルダを指定キーワードで検索し、結果にデザインを適用して返す。
	 */
	class searchresult_view
	{
		var $entryName = '';
		var $targetFolders = array();
		var $inlineXslt = null;  // ここに文字列で直接xsltを与える事ができる。
		
		/* ---------------------------------------------------------------
		 * コンストラクタ。
		 */
		function __construct()
		{
		}
		
		/* ---------------------------------------------------------------
		 * インラインXSLTを指定
		 * $xsltString : XSL文字列を設定。xml宣言とxsl:stylesheet要素は自動的に補完される。
		 */
		function setInlineXslt( $xsltString )
		{
			$this->inlineXslt = $xsltString;
		}
		
		/* ---------------------------------------------------------------
		 * インラインXSLTを補完後の形で取得
		 */
		function getInlineXsltFull()
		{
			$xslt = '<?xml version="1.0" encoding="UTF-8" ?>' . "\n" .
					'<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">' . "\n" . 
					'<xsl:output method="xml" encoding="' . gCMSDInternalEncoding() . '" omit-xml-declaration="yes" indent="no" />' . "\n";
					
			$xslt .= $this->inlineXslt;
			
			$xslt .= '</xsl:stylesheet>';
			
			return $xslt;

		}
		
		/* ---------------------------------------------------------------
		 * ユーザーパラメータの追加。
		 * $entryName : エントリ名。
		 * $keyValue : 省略時は値無し（URLから取得）。
		 */
		function addTarget( $name, $title, $pageurl )
		{
			$this->targetFolders[ $name ] = array( $name, $title, $pageurl );
		}
		
		/* ---------------------------------------------------------------
		 * ユーザーパラメータの全消去。
		 */
		function clearTarget()
		{
			$this->targetFolders = array();
		}
		
		function output( $designName, $rows )
		{
			$p_encoding = getGETParam( "encoding", gCMSDInternalEncoding() ); // 省略されていた場合は、CMSDの文字コードと考える
			$p_words = getGETParam( "words", "" );
			$p_rows = getGETParam( "rows", 100 );
			
			switch ( mb_strtoupper($p_encoding) )
			{
				case "UTF-8":
				case "EUC-JP":
				case "SHIFT_JIS":
					$encoding = $p_encoding;
					break;
				
				default:
					$encoding = "EUC-JP";
					break;
			}
			
			$p_words = mb_substr( trim( $p_words ), 0, 100, $encoding );
			$p_words = mb_convert_encoding( $p_words, "UTF-8", $encoding );	// XML文字列との比較の為、UTF-8に変換
			$words = explode( " ", $p_words );
			
			// 引数が0の場合、URLパラメータを採用する。
			if ( $rows == 0 )
			{
				$rows = $p_rows;
			}
			
			// 出力用のDOMを生成。
			$xmlDocOutput = domxml_new_doc( '1.0' );
		
			// ルートノード'entrylist'を生成。
			$xmlOutputRoot = $xmlDocOutput->add_root( 'entrylist' );
			
			// 検索ワードを出力用DOMに設定。
			$xmlOutputSearchWord = $xmlDocOutput->create_element( 'searchword' );
			$xmlOutputSearchWord->set_content( $p_words );
			$xmlOutputRoot->append_child( $xmlOutputSearchWord );
			
			// 検索文字列が指定されていたら、検索処理を実行
			if ( strlen( $p_words ) > 0 )
			{
				$maxRows = (int)$rows;
				foreach( $this->targetFolders as $target )
				{
					list( $entryName, $title, $pageUrl ) = $target;
					$maxRows = $this->outputTarget( $xmlDocOutput, $words, $entryName, $title, $pageUrl, $designName, $maxRows );
					
					// 戻り値が文字列なら、エラーなのでそれを返して終了
					if ( is_string( $maxRows ) )
					{
						 return $maxRows;
					}
				}
			}
			
			// XSL変換処理
			// 通常のページ表示変換処理
			//$result = $xmlDocOutput->dump_mem( true, "UTF-8" );
			if ( $designName )
			{
				$result = ADXslt::process( $xmlDocOutput, App::createListXslFilePath( "cmsd_system", $designName) );
			}
			else
			{
				$result = ADXslt::process( $xmlDocOutput, $this->getInlineXsltFull(), gCMSDInternalEncoding(), true );	// 最後の true は、xsltが文字列として渡される事を意味する。
			}
			
			$xmlDocOutput->free();
			return $result;
			
		}
		
		/* ---------------------------------------------------------------
		 * 検索の実行および、結果の表示。
		 * $entryName : エントリ名。
		 * $title : タイトル用項目名。
		 * $pageurl : エントリ表示用ページのURL。
		 * $designName : XSLスタイルシート名。
		 */
		function outputTarget( &$xmlDocOutput, $searchWords, $entryName, $title, $pageUrl, $designName, $maxRows )
		{
			/*	
			// Configの読み込み
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $entryName );
			
			// スキーマ名の取得
			$schemaName = cmsview_common::getSchemaName( $entryName );
			
			// スキーマ情報を配列形式で取得
			$schemaInfoArr = getSchemaInfoByArray( $schemaName );
			*/
			
			// 指定のデザインファイルが存在しない場合、エラーメッセージを表示して終了する。
			// 但し、designNameが指定されていなくても、inlineXsltが指定されている場合は問題なし。
			if ( strlen( $designName ) > 0 )
			{
			
				if ( ! file_exists( App::createListXslFilePath( "cmsd_system", $designName ) ))
				{
					return "ファイル'" . App::createListXslFileName( "cmsd_system", $designName ) . "'が見つかりません。又は埋め込みコマンドのdesign属性の指定が間違っています。";
				}
			
			}
			elseif ( strlen( $this->inlineXslt ) <= 0 )
			{
				return "埋め込みコマンドのdesign属性、又はcmsd:design要素がありません。";
			}
			
			// indexDOMを生成
			if ( file_exists( App::createEntryFilePath( $entryName, 'index' ) ) )
			{
				// ファイルを読み込む。
		   	 	$xmlDocIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
			}
			else
			{
				// indexファイルが無い場合、ここで空のindexDOMを生成する。
				$xmlDocIndex = domxml_new_doc( '1.0' );
			
				// ルートノード'entry'を生成。
				$xmlDocIndex->add_root( 'entrylist' );
			}
			
			// インデックス用のXPathの生成
			$xpathIndex = $xmlDocIndex->xpath_new_context();
			$xmlIndexRoot = $xmlDocIndex->document_element();
			
			/*
			// グループ条件がnullで指定されていた場合（ 'key'=>null ）、グループ指定をパラメータから読み込む。
			cmsview_common::setGroupKeyValue( $groupKeyValue );
			
			// グループ絞込み用のXPath部分文字列生成
			$cond = cmsview_common::createXPathCondition( $groupKeyValue, $entryConfig->publishControl );
			
			// グループとエントリ範囲による絞込み
			$xmlEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" . $cond . "[position()>=" . $startPos . " and position()<=" . $endPos . "]");
			*/
			
			// Configの読み込み
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $entryName );
			
			// グループ絞込み用のXPath部分文字列生成
			$cond = cmsview_common::createXPathCondition( null, $entryConfig->publishControl );
			
			// インデックスのエントリ参照要素一覧を取得
			$xmlEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry". $cond );
			
			// 出力用DOMのルート要素を取得
			$xmlOutputRoot = $xmlDocOutput->document_element();
			
			foreach( cmsd_get_nodeset( $xmlEntryRefArr ) as $entryIndex=>$xmlEntryRef )
			{
				
				// エントリDOMを取得する。
				$href = $xmlEntryRef->get_attribute('href');
				$xmlDocEntry = domxml_open_file( App::createEntryDirPath( $entryName ) . $href );
				
				// エントリデータを文字列で取得する
				$entryData = $xmlDocEntry->dump_mem( false, 'UTF-8' );
				
				// タグ除去
				$entryData = preg_replace( "/<[\/\!]*?[^<>]*?>/u", "", $entryData );
				
				// アンエスケープ
				//$entryData = html_entity_decode( $entryData, ENT_QUOTES, "UTF-8" );
				$entryData = unhtmlentities( $entryData );
				
				// 再度タグ除去
				$entryData = preg_replace( "/<[\/\!]*?[^<>]*?>/u", "", $entryData );
				
				// エスケープ
				//$entryData = str_replace( "&", "&amp;", $entryData );
				$entryData = htmlspecialchars( $entryData, ENT_QUOTES, "UTF-8" );
				
				//print $entryData;
				
				// 検索ワードに全てのキーワードがヒットするか？
				$hit = true;
				foreach( $searchWords as $searchWord )
				{
					$pos = mb_strpos( $entryData, $searchWord, 0, "UTF-8" );
					if ( $pos === false )
					{
						// 全て半角にして再度検索
						$pos = mb_stripos( mb_convert_kana($entryData, "askh", "UTF-8"), mb_convert_kana($searchWord, "askh", "UTF-8"), 0, "UTF-8");
					}
					if ( $pos === false )	// 複数の検索ワードのうち、１つでもヒットしなければ検索結果には追加しない
					{
						$hit = false;
						break;
					}
				}
				
				// ヒットしたら、検索結果に追加する。
				if ( $hit )
				{
					// エントリ要素を生成
					$xmlOutputEntry = $xmlDocOutput->create_element('entry');
				
					// エントリタイトル文字列を取得する
					$xmlEntryRoot = $xmlDocEntry->document_element();
					$xmlEntryTitleArr = $xmlEntryRoot->get_elements_by_tagname( $title );
					if ( empty( $xmlEntryTitleArr ) )
					{
						return "cmsd:searchtargetタグのtitle属性に指定された項目が見つかりません。";
					}
					$titleString = $xmlEntryTitleArr[0]->get_content();
				
					// 出力エントリにhref属性を追加する。
					$xmlOutputEntry->set_attribute('id', $xmlEntryRoot->get_attribute('id'));
					
					// 出力エントリにhref属性を追加する。
					$xmlOutputEntry->set_attribute('href', cmsview_common::createUrlParamsForEntry( $xmlEntryRoot->get_attribute('id'), null ));
					
					// 出力エントリにhref属性を追加する。
					$xmlOutputEntry->set_attribute('url', $pageUrl . cmsview_common::createUrlParamsForEntry( $xmlEntryRoot->get_attribute('id'), null ));
					
					// 出力エントリにtitleを追加する
					$xmlOutputEntryTitle = $xmlDocOutput->create_element("title");
					$xmlOutputEntryTitle->set_content( $titleString );
					$xmlOutputEntry->append_child( $xmlOutputEntryTitle );
					
					// 出力エントリにbodyを追加する
					$xmlOutputEntryBody = $xmlDocOutput->create_element("body");
					$xmlOutputEntryBody->set_content( $entryData );
					$xmlOutputEntry->append_child( $xmlOutputEntryBody );
					
					// 出力エントリを出力へ追加する。
					$xmlOutputRoot->append_child( $xmlOutputEntry );
					
					// 残り出力数を減らす。
					$maxRows--;
				}
				
				//unset( $xmlEntryRoot );
				//unset( $xmlDocEntry );
				
				$xmlDocEntry->free();
				
				// 残り出力数が0になったらループを抜ける
				if ( $maxRows <= 0 )
				{
					break;
				}
			}
			
			return $maxRows;
			
		}
		
	}		





	/* ---------------------------------------------------------------
	 * ビュークラス。
	 * スキーマ、エントリ、デザインを結びつけ、HTMLをレンダリングする。
	 */
 	class cmsview
	{

		/* ---------------------------------------------------------------
		 * エントリ1件の表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。（省略時は"default"）
		 * $entryId : エントリID。（省略時はREQUEST"eid"から取得）
		 * $showNavi : ナビゲーションを出力するかどうか。
		 * $isXML : XSLTを適用せず、XMLのまま出力するかどうか( true : XMLのまま出力する。 )
		 */
		function entry( $entryName, $designName = null, $entryId = '', $groupKeyValue = '', $showNavi = false, $isXML = false )
		{

			// リストオブジェクト生成
			$v = new entry_view( $entryName, $entryId );
			
			// グループ絞込み条件
			if ( is_array( $groupKeyValue ) )
			{
				foreach( $groupKeyValue as $key=>$value )
				{
					$v->addGroupKey( $key, $value );
				}
			}
			else
			{
				if ( strlen( $groupKeyValue ) > 0 )
				{
					$v->addGroupKey( "cmsd_singlekey", $groupKeyValue );	// "cmsd_singlekey"キー名は、旧仕様互換性の為の特別なキー名。単一キーとして処理される。
				}
			}
			
			// ナビゲーションのON/OFF
			if ( $showNavi )
			{
				$v->navigationOn();
			}
			else
			{
				$v->navigationOff();
			}
			
			// 出力形式の選択
			if ( $isXML )
			{
				// RSSの出力　又は　XMLの出力
				$v->outputxml( $designName );
			}
			else
			{
				// 通常のHTML出力
				$v->outputhtml( $designName );
			}
			
		}
		
		/* ---------------------------------------------------------------
		 * エントリ1件の表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。（省略時は"default"）
		 * $entryId : エントリID。（省略時はREQUEST"eid"から取得）
		 * $showNavi : ナビゲーションを出力するかどうか。
		 * $isXML : XSLTを適用せず、XMLのまま出力するかどうか( true : XMLのまま出力する。 )
		 */
		function old_entry( $entryName, $designName = null, $entryId = '', $groupKeyValue = '', $showNavi = false, $isXML = false )
		{
			// グループ条件がnullで指定されていた場合（ 'key'=>null ）、グループ指定をパラメータから読み込む。
			cmsview_common::setGroupKeyValue( $groupKeyValue );
			/*
			if ( is_array( $groupKeyValue ) )
			{
				foreach ( $groupKeyValue as $key=>$value )
				{
					if ( is_null($value) )
					{
						$groupKeyValue[$key] = $_REQUEST[$key];
					}
				}
			}
			*/
			
			// エントリIDが省略された場合、REQUESTから取得する。
			if ( empty( $entryId ) )
			{
				$entryId = $_REQUEST[ 'eid' ];
			}
			
			// REQUESTからも取得できない場合、"top"が指定されたものとみなす。
			if ( empty( $entryId ) ) 
			{
				$entryId = 'top';
			}
			
			// エントリIDに"top"が指定された場合、indexファイルの先頭エントリを適用する。
			if ( $entryId == 'top')
			{
				// indexDOMを生成
		   	 	$xmlDocIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
				$xpathIndex = $xmlDocIndex->xpath_new_context();
				 
				// グループによる絞込み
				if ( empty($groupKeyValue))
				{
					// グループキーが指定されていなければ、全て取得する。
					$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[position()=1]" );
				}
				else
				{
					// グループキーが指定されていたら、グループで絞り込む。
					$cond = cmsview_common::createXPathConditionForGroup( $groupKeyValue );
					if ( strlen( $cond ) > 0 )
					{
						$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[$cond][position()=1]" );
					}
					else
					{
						$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[position()=1]" );
					}
				}
				
				// 結果が存在すれば、そのエントリのIDを取得する。
				if ( !empty($xmlAllEntryRefArr->nodeset) )
				{
					$entryId = $xmlAllEntryRefArr->nodeset[0]->get_attribute('id');
				}
				
			}
			
		
			// スキーマ名の取得
			$schemaName = cmsview_common::getSchemaName( $entryName );

			// 指定のデザインファイルが存在しない場合、エラーメッセージを表示して終了する。
			if ( ! $isXML || $designName )	// XML指定じゃない場合は無条件でチェック、designNameが指定されていれば無条件でチェック。
			{	
				if ( ! file_exists( App::createXslFilePath( $schemaName, $designName ) ))
				{
					print "XSL file doesn't exists.'";
					return;
				}
			}
			

			// 指定のエントリファイルが存在しない場合、空っぽのエントリデータを生成する。
			if ( ! file_exists( App::createEntryFilePath( $entryName, $entryId ) ) )
			{
				// 空っぽのエントリデータ「noentry」要素を生成する。
				
				// DOMをを生成。
				$xmlDocEntry = domxml_new_doc( '1.0' );
				
				// ルートノード'noentry'を生成。
				$xmlEntryRoot = $xmlDocEntry->add_root( 'noentry' );
			}
			else 
			{
				// エントリが存在するなら、該当エントリを読み込んで出力用に加工する。
							
				// エントリDOMを生成
		   	 	$xmlDocEntry = domxml_open_file( App::createEntryFilePath( $entryName, $entryId ) );
	
				// DOMツリーの内容のtext/textarea項目を、出力用に調整する。
				trans_dom_for_output( $xmlDocEntry );
	
				// エントリDOMのルート要素を取得
				$xmlEntryRoot = $xmlDocEntry->document_element();
	
				// entryタグにhref属性を追加する。
				$xmlEntryRoot->set_attribute('href', cmsview_common::createUrlParamsForEntry( $entryId, $groupKeyValue ) );
				
				// entryタグにdaysago属性を追加する。
				set_daysago_attribute( $xmlEntryRoot );
			
				// ナビゲーション出力フラグがTrueならば、ナビゲーションを出力する。
				if ( $showNavi )
				{
					// indexDOMを生成
			   	 	$xmlDocIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
					$xpathIndex = $xmlDocIndex->xpath_new_context();
					 
					// グループによる絞込み
					if ( empty($groupKeyValue))
					{
						// グループキーが指定されていなければ、全て取得する。
						$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
					}
					else
					{
						// グループキーが指定されていたら、グループで絞り込む。
						$cond = cmsview_common::createXPathConditionForGroup( $groupKeyValue );
						if ( strlen( $cond ) > 0 )
						{
							$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[$cond]" );
						}
						else
						{
							$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
						}
					}
				
					// グループ内全件数
					$allEntryCount = count( cmsd_get_nodeset( $xmlAllEntryRefArr ) );
				
					// カレント位置
					$entryPos = -1;	// 初期値
					foreach ( cmsd_get_nodeset( $xmlAllEntryRefArr ) as $currEntryPos => $xmlEntryRef )
					{
						$currEntryId = $xmlEntryRef->get_attribute('id');
						//print $currEntryId . "=$entryId, ";
						if ( $currEntryId == $entryId )
						{
							$entryPos = $currEntryPos;
							break;
						}
					}
			
					if ( $entryPos >= 0 )
					{
						// navi要素を生成
						$xmlNavi = $xmlDocEntry->create_element('navi');
						$xmlNavi->set_attribute('position', $entryPos );
						$xmlNavi->set_attribute('allcount', $allEntryCount );
						
						// 前エントリが存在すれば、prev要素を追加
						if ( $entryPos > 0	)
						{
							$xmlPrev = $xmlDocEntry->create_element('prev');
							$id = $xmlAllEntryRefArr->nodeset[$entryPos-1]->get_attribute('id');
							$xmlPrev->set_attribute('id', $id );
							$xmlPrev->set_attribute('href', cmsview_common::createUrlParamsForEntry( $id, $groupKeyValue ) );
							$xmlNavi->append_child( $xmlPrev );
						}
					
						// 次ページが存在すれば、next要素を追加
						if ( $allEntryCount > $entryPos + 1 )
						{
							$xmlNext = $xmlDocEntry->create_element('next');
							$id = $xmlAllEntryRefArr->nodeset[$entryPos+1]->get_attribute('id');
							$xmlNext->set_attribute('id', $id );
							$xmlNext->set_attribute('href', cmsview_common::createUrlParamsForEntry( $id, $groupKeyValue ) );
							$xmlNavi->append_child( $xmlNext );
						}
					
						$xmlEntryRoot->append_child( $xmlNavi );
					}
					
				}
			}
			
			// グループが指定されていた場合、グループ情報を出力する。
			if ( ! empty( $groupKeyValue ) )
			{
				cmsview_common::addGroupInfo( $groupKeyValue, $xmlEntryRoot );
			}
			
			//print $xmlDocEntry->dump_mem( true, 'EUC-JP');
			
			/*
			$xslt = domxml_xslt_stylesheet_file( App::createXslFilePath( $schemaName, $designName) );
			$result = $xslt->process( $xmlDocEntry );
			print $result->dump_mem( true, 'EUC-JP' );
			*/
			
			if ( $isXML )
			{
				if ( $designName )
				{
					// XML出力はUTF-8固定
					$result = ADXslt::process( $xmlDocEntry, App::createXslFilePath( $schemaName, $designName), 'UTF-8' );
				}
				else
				{
					// XML出力はUTF-8固定
					$result = $xmlDocEntry->dump_mem( true, 'UTF-8' );
				}
			}
			else
			{
				$result = ADXslt::process( $xmlDocEntry, App::createXslFilePath( $schemaName, $designName) );
			}
			
			print $result;

			
		}
		
		
		
		/* ---------------------------------------------------------------
		 * エントリ1件の表示（ナビゲーション付き）。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。（省略時は"default"）
		 * $entryId : エントリID。（省略時はREQUEST"eid"から取得）
		 */
		function navi_entry( $entryName, $designName = 'default', $groupKeyValue = '' )
		{
			cmsview::entry( $entryName, $designName, "", $groupKeyValue, true );
		}
		
		
		/* ---------------------------------------------------------------
		 * エントリ1件の表示（ナビゲーション無し）。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。（省略時は"default"）
		 * $entryId : エントリID。（省略時はREQUEST"eid"から取得）
		 */
		function xml_entry( $entryName, $designName = null, $entryId = '', $groupKeyValue = '' )
		{
			cmsview::entry( 
				$entryName, 
				$designName, 			
				$entryId, 					// $entryId
				$groupKeyValue, 
				false, 					// $showNavi
				true 					// $isXML
			);
		}
		
		
		/* ---------------------------------------------------------------
		 * エントリ1件の表示（ナビゲーション付き）。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。（省略時は"default"）
		 * $entryId : エントリID。（省略時はREQUEST"eid"から取得）
		 */
		function xml_navi_entry( $entryName, $designName = null, $entryId = '', $groupKeyValue = '' )
		{
			cmsview::entry( 
				$entryName, 
				$designName, 			
				$entryId, 					// $entryId
				$groupKeyValue, 
				true, 					// $showNavi
				true 					// $isXML
			);
		}
		
		
		/* ---------------------------------------------------------------
		 * エントリリストの表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。
		 * $pageMaxRows : 1ページ当たりの行数。（省略値=10）
		 * $groupKeyValue : グループキー。（省略値=キーなし）
		 * $showNavi : ナビゲーションを出力するかどうか。
		 * $pageNo : ページ番号。（省略時はGETパラメータ"pageno"から取得）
		 * $isRSS : RSSかどうか。（省略時はfalse）
		 * $isXML : XMLとして出力するか。 (省略時はfalse)
		 */
		function listpage( $entryName, $designName = null, $pageMaxRows = 10, $groupKeyValue = '', $showNavi = true, $pageNo = NULL, $isRSS = false, $isXML = false )
		{
			// リストオブジェクト生成
			$v = new entrylist_view( $entryName, $pageNo );
			
			// グループ絞込み条件
			if ( is_array( $groupKeyValue ) )
			{
				foreach( $groupKeyValue as $key=>$value )
				{
					$v->addGroupKey( $key, $value );
				}
			}
			else
			{
				if ( strlen( $groupKeyValue ) > 0 )
				{
					$v->addGroupKey( "cmsd_singlekey", $groupKeyValue );	// "cmsd_singlekey"キー名は、旧仕様互換性の為の特別なキー名。単一キーとして処理される。
				}
			}
			
			// ナビゲーションのON/OFF
			if ( $showNavi )
			{
				$v->navigationOn();
			}
			else
			{
				$v->navigationOff();
			}
			
			// 出力形式の選択
			if ( $isRSS or $isXML )
			{
				// RSSの出力　又は　XMLの出力
				$v->outputxml( $designName, $pageMaxRows );
			}
			else
			{
				// 通常のHTML出力
				$v->outputhtml( $designName, $pageMaxRows );
			}
		}

		/* ---------------------------------------------------------------
		 * エントリリストの表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。
		 * $pageMaxRows : 1ページ当たりの行数。（省略値=10）
		 * $groupKeyValue : グループキー。（省略値=キーなし）
		 * $showNavi : ナビゲーションを出力するかどうか。
		 * $pageNo : ページ番号。（省略時はGETパラメータ"pageno"から取得）
		 * $isRSS : RSSかどうか。（省略時はfalse）
		 * $isXML : XMLとして出力するか。 (省略時はfalse)
		 */
		function old_listpage( $entryName, $designName = null, $pageMaxRows = 10, $groupKeyValue = '', $showNavi = true, $pageNo = NULL, $isRSS = false, $isXML = false )
		{
			
			// ページ番号の取得
			if ( empty($pageNo) )
			{
				$pageNo = $_REQUEST['pageno'];
			}
			if ( empty($pageNo) or !is_numeric($pageNo) )
			{
				$pageNo = 1;
			}
			
			// スキーマ名の取得
			$schemaName = cmsview_common::getSchemaName( $entryName );

			// 指定のデザインファイルが存在しない場合、エラーメッセージを表示して終了する。
			if ( ! $isXML || $designName )	// XML指定じゃない場合は無条件でチェック、designNameが指定されていれば無条件でチェック。
			{
				if ( ! file_exists( App::createListXslFilePath( $schemaName, $designName ) ))
				{
					print "XSL file doesn't exists.'";
					return;
				}
			}
			
			// indexDOMを生成
			if ( file_exists( App::createEntryFilePath( $entryName, 'index' ) ) )
			{
				// ファイルを読み込む。
		   	 	$xmlDocIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
			}
			else
			{
				// indexファイルが無い場合、ここで空のindexDOMを生成する。
				$xmlDocIndex = domxml_new_doc( '1.0' );
			
				// ルートノード'entry'を生成。
				$xmlDocIndex->add_root( 'entrylist' );
			}
			
			$xpathIndex = $xmlDocIndex->xpath_new_context();
			$xmlIndexRoot = $xmlDocIndex->document_element();
			
			// エントリ範囲の決定
			$startPos = $pageMaxRows * ( $pageNo - 1) + 1;
			$endPos = $pageMaxRows * ( $pageNo );
			
			// グループ条件がnullで指定されていた場合（ 'key'=>null ）、グループ指定をパラメータから読み込む。
			cmsview_common::setGroupKeyValue( $groupKeyValue );
			/*
			if ( is_array( $groupKeyValue ) )
			{
				foreach ( $groupKeyValue as $key=>$value )
				{
					if ( is_null($value) )
					{
						$groupKeyValue[$key] = $_REQUEST[$key];
					}
				}
			}
			*/
			
			// グループによる絞込み
			if ( empty($groupKeyValue))
			{
				// グループキーが指定されていなければ、全て取得する。
				$xmlEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[position()>=" . $startPos . " and position()<=" . $endPos . "]");
			}
			else
			{
				// グループキーが指定されていたら、グループで絞り込む。
				$cond = cmsview_common::createXPathConditionForGroup( $groupKeyValue );
				//print $cond;
				if ( strlen( $cond) > 0 )
				{
					$xmlEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[" . $cond . "][position()>=" . $startPos . " and position()<=" . $endPos . "]" );
				}
				else
				{
					$xmlEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[position()>=" . $startPos . " and position()<=" . $endPos . "]" );
				}
			
			}
			
			// 出力用DOMを生成
			//$xmlDocOutput = domxml_new_doc( '1.0' );
			//$xmlOutputRoot = $xmlDocOutput->add_root( 'entrylist' );
			
			$strXmlOutput = '<?xml version="1.0" encoding="UTF-8"?>'. "\n";
			$strXmlOutput = "<entrylist>\n";
			
			foreach( cmsd_get_nodeset( $xmlEntryRefArr ) as $xmlEntryRef )
			{
				// エントリDOMを取得する。
				$href = $xmlEntryRef->get_attribute('href');
				$xmlDocEntry = domxml_open_file( App::createEntryDirPath( $entryName ) . $href );
				$xmlEntryRoot = $xmlDocEntry->document_element();
				
				// エントリにhref属性を追加する。
				$xmlEntryRoot->set_attribute('href', cmsview_common::createUrlParamsForEntry( $xmlEntryRoot->get_attribute('id'), $groupKeyValue ));
				
				// エントリにdaysago属性を追加する。
				set_daysago_attribute( $xmlEntryRoot );
				
				// エントリをコピーして、出力用DOMにappendする。
				//$xmlOutputEntry = $xmlEntryRoot->clone_node();
				//$xmlOutputRoot->append_child( $xmlOutputEntry );
				
				$strXmlOutput .= $xmlDocEntry->dump_node( $xmlEntryRoot );
				
			}
			
			$strXmlOutput .= "</entrylist>";
			
			// 出力用XMLをDOM化
			$xmlDocOutput = domxml_open_mem( $strXmlOutput );

			//print '<pre>' . htmlentities( mb_convert_encoding( $xmlDocOutput->dump_mem(true, 'UTF-8'), 'EUC-JP' , 'UTF-8' ), ENT_COMPAT, 'EUC-JP' )	. '</pre>';
						
			// 出力用DOMのテキスト項目を出力用に変換
			trans_dom_for_output( $xmlDocOutput );
			
			// ナビゲーションの追加
			if ( $showNavi )
			{
				// グループによる絞込み
				if ( empty($groupKeyValue))
				{
					// グループキーが指定されていなければ、全て取得する。
					$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
				}
				else
				{
					// グループキーが指定されていたら、グループで絞り込む。
   					$cond = cmsview_common::createXPathConditionForGroup( $groupKeyValue );
					if ( strlen( $cond ) > 0 )
					{
						$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[" . $cond . "]" );
					}
					else
					{
						$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
					}
				}
				
				// ページ内のエントリ数
				$pageEntryCount = count( cmsd_get_nodeset( $xmlEntryRefArr ) );
				// グループ内の全エントリ数
				$allEntryCount = count( cmsd_get_nodeset( $xmlAllEntryRefArr ) );
				// 最大ページ数
				$maxPage = ( $allEntryCount / $pageMaxRows ) + (( $allEntryCount % $pageMaxRows > 0 ) ? 1 : 0 );
				
				$xmlOutputRoot = $xmlDocOutput->document_element();
				
				// navi要素を生成
				$xmlNavi = $xmlDocOutput->create_element('navi');
				$xmlNavi->set_attribute('position', $pageNo );
				$xmlNavi->set_attribute('allcount', $allEntryCount );
				$xmlNavi->set_attribute('maxpage', $maxPage );
				$xmlNavi->set_attribute('maxpagerows', $pageMaxRows );
				
				// 前ページが存在すれば、prev要素を追加
				if ( $pageNo > 1  )
				{
					$xmlPrev = $xmlDocOutput->create_element('prev');
					$xmlPrev->set_attribute('id', $pageNo - 1 );
					$xmlPrev->set_attribute('href', cmsview_common::createUrlParamsForPage( $pageNo - 1, $groupKeyValue ) );
					$xmlNavi->append_child( $xmlPrev );
				}
				
				// 存在するページ数分、page要素を追加
				for ( $index = 1; $index <= $maxPage; $index++ )
				{
					$xmlPage = $xmlDocOutput->create_element('page');
					$xmlPage->set_attribute('id', $index );
					$xmlPage->set_attribute('href', cmsview_common::createUrlParamsForPage( $index, $groupKeyValue ) );
					$xmlNavi->append_child( $xmlPage );
				}
				
				// 次ページが存在すれば、next要素を追加
				if ( $allEntryCount > $pageMaxRows * $pageNo )
				{
					$xmlNext = $xmlDocOutput->create_element('next');
					$xmlNext->set_attribute('id', $pageNo + 1 );
					$xmlNext->set_attribute('href', cmsview_common::createUrlParamsForPage( $pageNo + 1, $groupKeyValue ) );
					$xmlNavi->append_child( $xmlNext );
				}
				
				$xmlOutputRoot->append_child( $xmlNavi );
				
			}
			
			// グループが指定されていた場合、グループ情報を出力する。
			if ( ! empty( $groupKeyValue ) )
			{
				cmsview_common::addGroupInfo( $groupKeyValue, $xmlDocOutput->document_element() );
			}
			
			// XSL変換処理
			if ( $isRSS )
			{
				$result = ADXslt::process( $xmlDocOutput, App::createListXslFilePath( $schemaName, $designName), 'UTF-8' );
			}
			// XMLのまま出力（XSL変換なし）
			else if ( $isXML )
			{
				// デザイン定義名nullの場合はXSL変換なし。
				if ( $designName )
				{
					// XML出力はUTF-8固定
					$result = ADXslt::process( $xmlDocOutput, App::createListXslFilePath( $schemaName, $designName), 'UTF-8' );
				}
				else
				{
					// XML出力はUTF-8固定
					$result = $xmlDocOutput->dump_mem( true, 'UTF-8' );	
				}
			}
			// 通常のページ表示変換処理
			else
			{
				$result = ADXslt::process( $xmlDocOutput, App::createListXslFilePath( $schemaName, $designName) );
			}
			print $result;
			
			/*
			$xslt = domxml_xslt_stylesheet_file( App::createListXslFilePath( $schemaName, $designName) );
			$result = $xslt->process( $xmlDocOutput );
			print $result->dump_mem( true, 'EUC-JP' );
			*/
			
			/*
			//$xmlXsl = domxml_open_file( App::createListXslFilePath( $schemaName, $designName) );
			
			$strXml = $xmlDocOutput->dump_mem( true, 'UTF-8' );
			//$strXsl = $xmlXsl->dump_mem( true, 'UTF-8' );
			
			//print "XML string : '<pre>" . htmlconv( $strXml ) . "</pre>'";
			//print "XSL string : '<pre>" . htmlconv( $strXsl ) . "</pre>'";
			
			$xslt = xslt_create();
			$result = xslt_process( $xslt, 'arg:/_xml', App::createListXslFilePath( $schemaName, $designName), NULL, array( '/_xml' => $strXml ) );

			xslt_free( $xslt );
			
			// EUC-JPに変換し、出力する。
			print $result;
			//print( mb_convert_encoding( $result, "EUC-JP", "UTF-8" ) ); 
			*/
			
		}

		/* ---------------------------------------------------------------
		 * エントリ1件の表示。
		 * エントリリストの表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。
		 * $rows : 表示行数。（省略値=10）
		 * $groupKeyValue : グループキー。（省略値=キーなし）
		 */
		function listtop( $entryName, $designName, $rows = 5, $groupKeyValue = '' )
		{
			cmsview::listpage( $entryName, $designName, $rows, $groupKeyValue, false, 1 );
		}

		/* ---------------------------------------------------------------
		 * エントリリストの表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。
		 * $pageMaxRows : 1ページ当たりの行数。（省略値=10）
		 * $groupKeyValue : グループキー。（省略値=キーなし）
		 * $pageNo : ページ番号。（省略時はGETパラメータ"pageno"から取得）
		 * $isRSS : RSSかどうか。（省略時はfalse）
		 * $isXML : XMLとして出力するか。 (省略時はfalse)
		 */
		function xml_listpage( $entryName, $designName = null, $pageMaxRows = 10, $groupKeyValue = '', $pageNo = NULL )
		{
			cmsview::listpage( 
				$entryName, 
				$designName, 
				$pageMaxRows, 
				$groupKeyValue, 
				true,					// $showNavi
				$pageNo, 				// $pageNo
				false, 					// $isRSS
				true 					// $isXML
			);
		}
				
		/* ---------------------------------------------------------------
		 * エントリ1件の表示。
		 * エントリリストの表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。
		 * $rows : 表示行数。（省略値=10）
		 * $groupKeyValue : グループキー。（省略値=キーなし）
		 */
		function xml_listtop( $entryName, $designName = null, $rows = 5, $groupKeyValue = '' )
		{
			cmsview::listpage( 

				$entryName, 
				$designName, 
				$rows, 
				$groupKeyValue, 
				false, 		// $showNavi 
				1, 			// $pageNo
				false,		// $isRSS
				true 		// $isXML
			);
		}
		
		/* ---------------------------------------------------------------
		 * エントリ1件の表示。
		 * エントリリストの表示。
		 * $entryName : エントリ名。
		 * $designName : XSLスタイルシート名。
		 * $groupKeyValue : グループキー。（省略値=キーなし）
		 */
		function rss( $entryName, $designName, $groupKeyValue = '' )
		{
			cmsview::listpage( $entryName, $designName, 15, $groupKeyValue, false, 1, true );
		}
		
		
		
		
		
		function entrynavi( $entryName, $groupKeyValue = '', $entryId = NULL )
		{
			// エントリIDの取得
			if ( empty($entryId) )
			{
				$entryId = $_REQUEST['eid'];
			}
	
			// indexDOMを生成
	   	 	$xmlDocIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
			$xpathIndex = $xmlDocIndex->xpath_new_context();
			
			// グループによる絞込み
			if ( empty($groupKeyValue))
			{
				// グループキーが指定されていなければ、全て取得する。
				$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
			}
			else
			{
				// グループキーが指定されていたら、グループで絞り込む。
				$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[groupkey='" . $groupKeyValue . "']" );
			}
			
			// グループ内全件数
			$allEntryCount = count( cmsd_get_nodeset( $xmlAllEntryRefArr ) );
			
			// カレント位置
			$entryPos = -1;	// 初期値
			foreach ( cmsd_get_nodeset( $xmlAllEntryRefArr ) as $currEntryPos => $xmlEntryRef )
			{
				$currEntryId = $xmlEntryRef->get_attribute('id');
				//print $currEntryId . ", ";
				if ( $currEntryId == $entryId )
				{
					$entryPos = $currEntryPos;
					break;
				}
			}
		
			if ( $entryPos >= 0 )
			{
				if ( $entryPos > 0	)
				{
					print '<a href="?eid=' . $xmlAllEntryRefArr->nodeset[$entryPos-1]->get_attribute('id') . '">前のエントリ</a> ';
				}
			
				print ($entryPos + 1) . '番目のエントリ（全 ' . $allEntryCount . '件） ';
				
				if ( $allEntryCount > $entryPos + 1 )
				{
					print ' <a href="?eid=' . $xmlAllEntryRefArr->nodeset[$entryPos+1]->get_attribute('id') . '">次のエントリ</a>';
				}
			
				print '<br />';
			}
			else
			{
				print "Such entryId doesn't exists.<br />";
			}
			
		}
		
		
		function pagenavi( $entryName, $pageMaxRows = 10, $groupKeyValue = '', $pageNo = NULL )
		{
			// ページ番号の取得
			if ( empty($pageNo) )
			{
				$pageNo = $_REQUEST['pageno'];
			}
			if ( empty($pageNo) or !is_numeric($pageNo) )
			{
				$pageNo = 1;
			}
			
			// 指定のindexファイルが存在しない場合、エラーメッセージを表示して終了する。
			if ( ! file_exists( App::createEntryFilePath( $entryName, 'index' ) ) )
			{
				print "Index file doesn't exists.'";
				return;
			}
			
			// indexDOMを生成
	   	 	$xmlDocIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
			$xpathIndex = $xmlDocIndex->xpath_new_context();
			
			// エントリ範囲の決定
			$startPos = $pageMaxRows * ( $pageNo - 1) + 1;
			$endPos = $pageMaxRows * ( $pageNo );
			
			// グループによる絞込み
			if ( empty($groupKeyValue))
			{
				// グループキーが指定されていなければ、全て取得する。
				$xmlPageEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[position()>=" . $startPos . " and position()<=" . $endPos . "]");
				$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry" );
			}
			else
			{
				// グループキーが指定されていたら、グループで絞り込む。
				$xmlPageEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[groupkey='" . $groupKeyValue . "'][position()>=" . $startPos . " and position()<=" . $endPos . "]" );
				$xmlAllEntryRefArr = $xpathIndex->xpath_eval( "/entrylist/entry[groupkey='" . $groupKeyValue . "']" );
			}
			
			
			$pageEntryCount = count( cmsd_get_nodeset( $xmlPageEntryRefArr ) );
			$allEntryCount = count( cmsd_get_nodeset( $xmlAllEntryRefArr ) );
			
			if ( $pageNo > 1  )
			{
				print '<a href="?pageno=' . ( $pageNo - 1 ) . '">前のページ</a> ';
			}
			
			print $pageNo . 'ページ（全 ' . $allEntryCount . '件） ';
			
			if ( $allEntryCount > $pageMaxRows * $pageNo )
			{
				print ' <a href="?pageno=' . ( $pageNo + 1 ) . '">次のページ</a>';
			}
			
			print '<br />';
			
		}
		
		
	}
	
	/* ---------------------------------------------------------------
	 * エントリノードから日付を取得し、「今日」との日数差を算出してdaysago属性として追加する。
	 * $xmlEntry : エントリのノード。
	 */	
	function set_daysago_attribute( &$xmlEntry )
	{
		$year = intval( $xmlEntry->get_attribute('year') );
		$month = intval( $xmlEntry->get_attribute('month') );
		$day = intval( $xmlEntry->get_attribute('day') );
		$hour = intval( $xmlEntry->get_attribute('hour') );
		$minute = intval( $xmlEntry->get_attribute('minute') );
		
		// エントリ日付
		$dtEntry = mktime( 0, 0, 0, $month, $day, $year );	// 時刻情報は含めずに計算する
		// 今日の日付
		$dtNow = mktime(0, 0, 0);	// 時刻情報は含めずに計算する。
		
		$daysago = intval(($dtNow - $dtEntry) / ( 24 * 60 * 60 ));
		
		$xmlEntry->set_attribute('daysago', $daysago );
		
		// time属性が追加されていない場合、ここで追加する。
		if ( $xmlEntry->get_attribute('time') === '' )
		{
			$xmlEntry->set_attribute('time', mktime( $hour, $minute, 0, $month, $day, $year ) );
		}
		
	}
	
	/* ---------------------------------------------------------------
	 * エントリノードから日付を取得し、今日の日付情報を追加する。
	 * $xmlEntry : エントリのノード。
	 */	
	function set_today_info( &$xmlEntry )
	{
		// 現在日時のtime値
		$dtNow = time();
	
		// 年月日時分秒
		$sYear = sprintf( "%04d", date( "Y", $dtNow ) );
		$sMonth = sprintf( "%02d", date( "m", $dtNow ) );
		$sDay = sprintf( "%02d", date( "d", $dtNow ) );
		$sHour = sprintf( "%02d", date( "H", $dtNow ) );
		$sMinute = sprintf( "%02d", date( "i", $dtNow ) );
		$sSecond = sprintf( "%02d", date( "s", $dtNow ) );
		$timezone_offset = date( "Z", $dtNow );
		
		// 曜日を計算する。
		$arrWeekday = array( 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' );
		$iWeekday = date('w', $dtNow );
		$sWeekday = $arrWeekday[ $iWeekday ];
		
		// 日時文字列の生成
		$strDate = $sYear . '-' . $sMonth . '-' . $sDay . ' ' . $sHour . ':' . $sMinute;
		
		// today要素の生成
		$doc = $xmlEntry->owner_document();
		$xmlToday = $doc->create_element( 'today' );
		
		$xmlToday->set_content( $strDate );
		$xmlToday->set_attribute( 'year', $sYear );
		$xmlToday->set_attribute( 'month', $sMonth );
		$xmlToday->set_attribute( 'day', $sDay );
		$xmlToday->set_attribute( 'hour', $sHour );
		$xmlToday->set_attribute( 'minute', $sMinute );
		$xmlToday->set_attribute( 'second', $sSecond );	
		$xmlToday->set_attribute( 'weekday', $sWeekday );
		$xmlToday->set_attribute( 'time', $dtNow );
		$xmlToday->set_attribute( 'tzoffset', $timezone_offset );
		
		$xmlEntry->append_child( $xmlToday );
	}
	
	/* ---------------------------------------------------------------
	 * DOMツリー内のメニュー項目に情報を付加する。ラベル情報や、外部データソースの子エントリ要素の埋め込み等。
	 * $xmlDocEntry : エントリのDOM。
	 * $schemaInfoArr : スキーマ情報配列。
	 * $entryConfig : エントリ定義。主にデータソースの取得の為。
	 */	
	function set_menuitem_info( &$xmlDocOutput, $schemaInfoArr, $entryConfig, $isRecursive = true )
	{
	
		// スキーマからメニュー項目だけを抽出して処理
		foreach( $schemaInfoArr as $id=>$schemaInfo )
		{
			// メニュー項目か？
			if ( $schemaInfo['type'] === 'menu' )
			{
				
				// 項目名に対応する外部データソースはsite.config.xml上に定義されているか？
				$datasource = $entryConfig->getDatasource( $id ); // ここで、idは項目名。
				
				// 子要素の埋め込みを行うか？：対応するデータソースのtypeがentrylistの場合に行う（引数でisRecursiveがfalseになっていた場合は強制的に行わない）
				$isEmbeddingEntry = ( $isRecursive && !empty( $datasource ) && ( $datasource->embedding ) && ( $datasource->type === 'entrylist' ) );
				
				$childEntryName = null;
				$childSchemaInfoArr = null;
				$childEntryConfig = null;
				if ( $isEmbeddingEntry )
				{
					// 子エントリのエントリ名
					$childEntryName = $datasource->from;
				
					// 子エントリのスキーマ情報を配列形式で取得
					$childSchemaInfoArr = getSchemaInfoByArray( cmsview_common::getSchemaName( $childEntryName ) );
					
					// 子エントリのエントリ定義を取得
					$siteConfig = new CMSSiteConfig();
					$childEntryConfig = $siteConfig->getEntryConfig( $childEntryName );
				}
				
				// メニュー項目選択肢の取得
				$menuItems = array();
				if ( !empty( $datasource ) ) {
					$menuItems = $datasource->menuItems;
				} else {
					$menuItems = $schemaInfo['menuItems'];
				}
				
				// 出力用DOMの子要素として<menuinfo name="ITEMNAME"><menuitem id="xx">ABC</menuitem></menuinfo>を追加する
				$xmlMenuInfo = $xmlDocOutput->create_element( 'menuinfo' );
				$xmlMenuInfo->set_attribute( 'name', $id ); // idは項目名
				foreach( $menuItems as $menuItemId => $menuItemValue )
				{
					$xmlMenuItem = $xmlDocOutput->create_element( 'menuitem' );
					$xmlMenuItem->set_content( fileconv( $menuItemValue ) );
					$xmlMenuItem->set_attribute( 'id', fileconv( $menuItemId ) );
					$xmlMenuInfo->append_child( $xmlMenuItem );
				}
				$xmlDocOutput->document_element()->append_child( $xmlMenuInfo );
				
				// このメニュー項目をエントリdocから全て抽出する（エントリ単一の場合とエントリリストの場合と両方で呼ばれる為、/entry/{$id} とは書けない）。
				$xpathOutput = $xmlDocOutput->xpath_new_context();
				$xmlOutputNodeArr = cmsd_get_nodeset( $xpathOutput->xpath_eval("//{$id}") );
				
				// 項目単位で処理
 				foreach ( $xmlOutputNodeArr as $xmlOutputNode )
				{
				
					$label = '';
					$menuItemId = $xmlOutputNode->get_content();
					
					// ※xmlNodeに対して変更を加えると、画面が真っ白になる模様。→ダブルバイト文字をそのまま指定していた事が原因だった。fileconvを通したら問題なくなった。
					
					if ( $isEmbeddingEntry )
					{
						// 対応する子エントリデータをdocとして取得し、importNodeで子要素を現在のエントリの子要素としてインポートする。
						$childEntryId = $menuItemId;
						
						// 子エントリDOMを生成
						$xmlDocChildEntry = domxml_open_file( App::createEntryFilePath( $childEntryName, $childEntryId ) );
						
						if ( !empty( $xmlDocChildEntry ) )
						{
							// 子エントリのテキスト項目をスキーマ情報に合わせて調整
							trans_dom_for_output( $xmlDocChildEntry, $childSchemaInfoArr );
							
							// 子エントリのメニュー項目をスキーマ情報に合わせて調整（子要素の再帰埋め込みは行わない）
							set_menuitem_info( $xmlDocChildEntry, $childSchemaInfoArr, $childEntryConfig, FALSE );
							
							// 出力用docにインポート
							$xmlOutputNode->myDOMNode->appendChild( $xmlOutputNode->myOwnerDocument->myDOMNode->importNode( $xmlDocChildEntry->document_element()->myDOMNode, TRUE ) ); // deep=TRUE: サブノードも再帰的にインポート
							
							/*
							$xmlChildEntry = $xmlDocOutput->create_element( 'childentry' );
							$xmlChildEntry->set_attribute( 'id', $menuItemId );
							$xmlChildEntry->set_attribute( 'message', fileconv( 'メッセージ' ) );
							$xmlOutputNode->append_child( $xmlChildEntry );
							*/
						}
						else {
							$xmlChildEntry = $xmlDocOutput->create_element( 'noentry' );
							$xmlOutputNode->append_child( $xmlChildEntry );
						}
					
					}
					
					// メニュー選択肢から、エントリデータに対応するラベルを取得
					if ( array_key_exists( $menuItemId, $menuItems ) )
					{
						$label = $menuItems[ $menuItemId ];
					}
					
					// エントリデータにラベルを設定（label属性）
					$xmlOutputNode->set_attribute( 'label', fileconv( $label ) );
				}
				
			}
		}
		
	}
	
	/* ---------------------------------------------------------------
	 * DOMツリー内の全てのテキスト項目を表示用に変換する。
	 * $xmlDocEntry : エントリのDOM。
	 * $tagRemoving : タグ除去するか。（省略値：false（しない））
	 */	
	function trans_dom_for_output( &$xmlDocEntry, $schemaInfoArr, $tagRemoving = false )
	{
		$xpathEntry = $xmlDocEntry->xpath_new_context();
		$xmlTextNodeArr = $xpathEntry->xpath_eval("//*[@output!='']");

		foreach ( cmsd_get_nodeset( $xmlTextNodeArr ) as $xmlTextNode )
		{
			$strText = $xmlTextNode->get_content();
			$outputMode = $xmlTextNode->get_attribute('output');
			
			// スキーマ情報の取得
			$tagname = $xmlTextNode->node_name();
			$schemaInfo = null;
			if ( isset( $schemaInfoArr[ $tagname ] ) )
			{
				$schemaInfo = $schemaInfoArr[ $tagname ];
				
				if ( isset( $schemaInfo[ 'output' ] ) )
				{
					// スキーマ情報があればそちらを優先する。
					$outputMode = $schemaInfo[ 'output' ];
				}
			}
			
			// タグ除去がONならば、タグ除去だけ行う。
			if ( $tagRemoving )
			{
				$strText = preg_replace( "/<[\/\!]*?[^<>]*?>/u", "", $strText );
				XmlUtil::deleteTextNode( $xmlTextNode );
				$strText = htmlspecialchars( $strText, ENT_QUOTES );
				$xmlTextNode->set_content( $strText );
			}
			else	// タグ除去がOFFならば、各種変換処理を行う。
			{
			
				switch ( $outputMode )
				{
					case 'text1' :
						
						break;
				
					case 'text2' :
						$strText = htmlentities( $strText, ENT_COMPAT, 'UTF-8' );
						$strText = htmlentities( $strText, ENT_COMPAT, 'UTF-8' );
						
						// URLの自動リンク（autolink指定の場合のみ） ※htmlentitiesの前に自動リンクすると、リンクそのものがエスケープされる為、ここで処理する。
						if ( isset( $schemaInfo[ 'autolink' ] ) and $schemaInfo[ 'autolink' ] == 'True' )
						{
							$strText = autolink_html( $strText );
						}
						
						$strText = nl2br( $strText ); //str_replace( "\n", '<br />&#13;', $strText );
						XmlUtil::deleteTextNode( $xmlTextNode );
						$xmlTextNode->set_content( $strText );
					
						break;
				
					case 'html1' :
					
						break;
					
					case 'html2' :
						// URLの自動リンク（autolink指定の場合のみ）
						if ( isset( $schemaInfo[ 'autolink' ] ) and $schemaInfo[ 'autolink' ] == 'True' )
						{
							$strText = autolink_html( $strText );
						}
						$strText = nl2br_html( $strText );
						XmlUtil::deleteTextNode( $xmlTextNode );
						$strText = htmlspecialchars( $strText, ENT_QUOTES );
						$xmlTextNode->set_content( $strText );
						break;
				
				}
			}
		}
	}
	
	// html_entity_decode関数の代わり（PHP4環境ではMBCSに対応していない為）
	function unhtmlentities($string)
	{
		// 数値エンティティの置換
		$string = preg_replace_callback('~&#x([0-9a-f]+);~i', function($m){ return chr(hexdec($m[1])); }, $string);
		$string = preg_replace_callback('~&#([0-9]+);~', function($m){ return chr($m[1]); }, $string);
		
		// 文字エンティティの置換
		$trans_tbl = get_html_translation_table(HTML_ENTITIES);
		$trans_tbl = array_flip($trans_tbl);
		
		return strtr($string, $trans_tbl);
	}	
	
	// 文字列中に出現する URL を自動的にリンクする（Aタグで囲む）。preg系を使っている為、UTF-8以外不可。
	function autolink_html( $strText )
	{
		// 2007.6.15 ereg系関数を使うとアクセス集中時に固まる(?)のを回避する為、preg系に置き換えた。
		$strText = preg_replace("/([^=\"']|^)(https?|ftp)(:\\/\\/[[:alnum:]\+\$\;\?\.%,!#~*\\/:@&=_-]+)/ui", "\\1<a href=\"\\2\\3\" >\\2\\3</a>", $strText);
		//$strText = ereg_replace("([^=\"']|^)(https?|ftp)(://[[:alnum:]\+\$\;\?\.%,!#~*/:@&=_-]+)", "\\1<a href=\"\\2\\3\" >\\2\\3</a>", $strText);
		
		return $strText;
	}
	
	// 文字列中に出現する改行に<br />をつける。一部のタグの直後の改行は変換しない。preg系を使っている為、UTF-8以外不可。
	function nl2br_html( $str )
	{
		$strArr = preg_split( "/(\n?<(?:pre|script|style|iframe|PRE|SCRIPT|STYLE|IFRAME)[^>]*>\n?|\n?<\/(?:pre|script|style|iframe|PRE|SCRIPT|STYLE|IFRAME)[^>]*>\n?)/u", $str, -1, PREG_SPLIT_DELIM_CAPTURE );
		//$strArr = preg_split( "/(\n?<pre[^>]*>\n?|\n?<\/pre[^>]*>\n?|\n?<PRE[^>]*>\n?|\n?<\/PRE[^>]*>\n?)/", $str, -1, PREG_SPLIT_DELIM_CAPTURE );

		$strResult = '';
		$countTextSections = 0;
		$mode = 'nopre';
		foreach ( $strArr as $tok )
		{
			//$tok = '(**' . $tok . '**)';	// トークンの切り出しチェック用デバッグ文
		
			//if ( preg_match( "/<pre[^>]*>|<PRE[^>]*>/", $tok ) )
			if ( preg_match( "/<(pre|script|style|iframe)[^>]*>/u", strtolower( $tok ) ) )
			{
				// pre,script,object,iframe開始タグを発見したら、モードをpreにする。
				$countTextSections ++;
				
				if ( $countTextSections > 0 )
				{
					$mode = 'pre';
				}
				$strResult .= $tok;
			}
			//else if ( preg_match( "/<\/pre[^>]*>|<\/PRE[^>]*>/", $tok ) )
			else if ( preg_match( "/<\/(pre|script|style|iframe)[^>]*>/u", strtolower( $tok ) ) )
			{
				// PRE終了タグを発見したら、モードをnopreにする。
				$countTextSections --;
				if ( $countTextSections < 0 ) $countTextSections = 0;
				
				if ( $countTextSections <= 0 )
				{
					$mode = 'nopre';
				}
				$strResult .= $tok;
			}
			else
			{
				if ( $mode == 'nopre' )
				{
					// nopre中は、改行部分に<br />を付け加える。
					// まず、改行をトークンにしてsplitする。
					$lineArr = preg_split( "/\n/", $tok );//, -1, PREG_SPLIT_DELIM_CAPTURE );
					if ( count($lineArr) == 1 )
					{
						// \nで区切って結果が1つだけ、ということは、\nが存在しなかったということなので、そのまま出力する。
						$strResult .= $lineArr[0];	
					}
					else
					{
						// <br />タグの付加処理を行う。
						foreach( $lineArr as $lineNo=>$lineStr )
						{
							// トークンの内容を出力する。
							$strResult .= rtrim($lineStr);
							
							if ( $lineNo != count($lineArr)-1 )
							{
								$needsBR = true;	// 基本は行末に <br />を出力する。
								
								if ( mb_substr( trim($lineStr), mb_strlen(trim($lineStr)) - 1, 1 ) == '>' )
								{
									// 行末が">"（タグで終わっている）場合は、<br />を出力しない。
									$needsBR = false;
								}
								
								// 但し、行末が ">" だったとしても、以下の閉じタグだった場合はやはり出力する。
								$matches = array();
								preg_match( "/<\/([^>]*)>$/u", trim($lineStr), $matches );
								if ( isset( $matches[1] ) )
								{
									switch ( strtolower( trim( $matches[1] ) ) )
									{
										case 'a' :
										case 'font' :
										case 'span' :
										case 'b' :
										case 'strong' :
										case 'i' :
										case 'em' :
										case 'u' :
										case 's' :
										case 'tt' :
										case 'small' :
										case 'sup' :
										case 'sub' :
										case 'big' :
											
											$needsBR = true;
											break;
									}
								}
								
								if ( $needsBR ) 
								{
									$strResult .= '<br />';
								}
								
								/*
								if ( mb_substr( trim($lineStr), mb_strlen(trim($lineStr)) - 1, 1 ) != '>' )
								{
									// 行末が">"（タグで終わっている）場合とトークンが改行文字の場合を除き、<br />を出力する。
									$strResult .= '<br />';
								}
								*/
							}
							
							$strResult .= "\n";
						}
					}
				//$strResult .= mb_ereg_replace( '[^>]($)', "<br />", $tok );  // ←最初これで一括置換しようとしたが、日本語が一部化けた。
				}
				else
				{
					$strResult .= $tok;
				}
			}
		
		  //print "token = " . $tok . "\n";
		}
		
		return $strResult;

	}
	

	/* ---------------------------------------------------------------
	 * ビュークラス。
	 * スキーマ、エントリ、デザインを結びつけ、HTMLをレンダリングする。
	 */
	class View2
	{
		var $name;					// ビュー名
		var $schemaName;			// スキーマ名
		var $desginName;			// デザイン名
		var $maxPageEntryCount; 	// 1ページ当たりの最大エントリ数
		
		
		/* -----------------------------------------------------------
		 * コンストラクタ。
		 */
		function __construct( $name )
		{
			$this->name = $name;
			$this->entryName = "news1";
			$this->designName = "news_all";
			$maxPageEntryCount = 10;
		}
		
		/* -----------------------------------------------------------
		 * HTMLのレンダリング
		 */
		function renderHTML( $entryID )
		{
			// HTMLヘッダの出力
			header("Content-Type:text/html;charset=EUC-JP");
			
			// ルートディレクトリパスを取得
			$strDir = App::getRootDir();
			
			// エントリのディレクトリパスを取得。
			$strEntryDir = $strDir . "/data/entry/" . $this->entryName . "/";
			
			// エントリXMLファイルパスを生成
			$strXml = $strEntryDir . $this->entryName . "." . $entryID . ".xml";
			
			// デザインXSLファイルのパスを生成
			$strXsl = $strDir . "/config/design/" . $this->designName . "/" .
				$this->designName . ".xsl";
			
			// XSLTを生成し、HTMLを生成する。
			$objPrc = xslt_create();
			xslt_set_base( $objPrc, "file://" . $strEntryDir ); 	// XSLTのBaseURIにエントリDirを指定。
			$result = xslt_process( $objPrc, $strXml, $strXsl );
			
			// EUC-JPに変換し、出力する。
			print( mb_convert_encoding( $result, "EUC-JP", "UTF-8" ) ); 	
		}
		
	}
	
	
	
	//-------------------------------------------------------------------------
	// array parse_http_date( string Date )
	// DateはRFC1123、RFC 850、ANSI C's asctime() formatのいずれか。
	//-------------------------------------------------------------------------
	function parse_http_date( $string_date ) {

		// 月の名前と数字を定義
		$define_month = array(
			"01" => "Jan", "02" => "Feb", "03" => "Mar",
			"04" => "Apr", "05" => "May", "06" => "Jun",
			"07" => "Jul", "08" => "Aug", "09" => "Sep",
			"10" => "Oct", "11" => "Nov", "12" => "Dec"
		);

		if( preg_match( "/^(Mon|Tue|Wed|Thu|Fri|Sat|Sun), ([0-3][0-9]) (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) ([0-9]{4}) ([0-2][0-9]):([0-5][0-9]):([0-5][0-9]) GMT$/", $string_date, $temp_date ) ) {

			$date["hour"] = $temp_date[5];
			$date["minute"] = $temp_date[6];
			$date["second"] = $temp_date[7];
			// 定義済みの月の名前を数字に変換する
			$date["month"] = array_search( $temp_date[3], $define_month );
			$date["day"] = $temp_date[2];
			$date["year"] = $temp_date[4];


		} elseif( preg_match( "/^(Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|Sunday), ([0-3][0-9])-(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)-([0-9]{2}) ([0-2][0-9]):([0-5][0-9]):([0-5][0-9]) GMT$/", $string_date, $temp_date ) ) {

			$date["hour"] = $temp_date[5];
			$date["minute"] = $temp_date[6];
			$date["second"] = $temp_date[7];
			// 定義済みの月の名前を数字に変換する
			$date["month"] = array_search( $temp_date[3], $define_month );
			// 年が2桁しかないので1900を足して4桁に
			$date["day"] = $temp_date[2];
			$date["year"] = 1900 + $temp_date[4];


		} elseif( preg_match( "/^(Mon|Tue|Wed|Thu|Fri|Sat|Sun) (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) ([0-3 ][0-9]) ([0-2][0-9]):([0-5][0-9]):([0-5][0-9]) ([0-9]{4})$/", $string_date, $temp_date ) ) {
			$date["hour"] = $temp_date[4];
			$date["minute"] = $temp_date[5];
			$date["second"] = $temp_date[6];
			$date["month"] = array_search( $temp_date[2], $define_month );
			// 日が1桁の場合先、半角スペースを0に置換
			$date["day"] = str_replace( " ", 0, $temp_date[3] );
			// 定義済みの月の名前を数字に変換する
			$date["year"] = $temp_date[7];


		} else {
			return FALSE;
		}

		// UNIXタイムスタンプを生成GMTなのに注意
		$date["timestamp"] = gmmktime( $date["hour"], $date["minute"], $date["second"], $date["month"], $date["day"], $date["year"] );

		return $date;

	}	
	
	// ファイルダウンロード
	class FileDownloader
	{
		function __construct()
		{
		}
		
		function output()
		{
			// パラメータの取得
			$entryName = $_GET['entryname'];
			$entryId = $_GET['entryid'];
			$fileid = $_GET['fileid'];
			$mode = getGETParam( 'mode' );	// 'manager'ならば、管理画面モード。
			$disp = getGETParam( 'disp' );	// 'inline'ならばダウンロードではなくブラウザに表示。
			
	   	 	$xmlDocEntry = domxml_open_file( App::createEntryFilePath( $entryName, $entryId ) );
			$xpathEntry = $xmlDocEntry->xpath_new_context();
			$xmlEntryDataArr = $xpathEntry->xpath_eval( "//*[@fileid='$fileid']" );
			
			//$srcFileName = $xmlEntryDataArr->nodeset[0]->get_attribute('src');
			$orgFileName = $xmlEntryDataArr->nodeset[0]->get_attribute('org');
			
			//$ext = getfileext( $orgFileName );
			
			//$srcFilePath = App::createAttachedFilePath( $entryName, $entryId, $fileid, $orgFileName );

			// 編集中のファイルを要求された場合、まだエントリデータ中に情報が保存されてないので、拡張子情報を実際のフォルダのファイルから探しあてる必要がある。
			// 拡張子「*」で検索して、ファイルリストを取得する。
			$srcFilePathDammy = App::createAttachedFilePath( $entryName, $entryId, $fileid, 'dammy.*' );
			$srcFilePathArr = glob( $srcFilePathDammy );
			
			if ( $mode == 'manager' )
			{
				// 管理画面モードの場合、更新日時が一番新しいファイルを該当ファイルとする。
				$lastmtime = 0;	// 一番新しいファイルの更新日時（初期値は0から開始）
				foreach ( $srcFilePathArr as $srcFilePathItem )
				{
					$mtime = filemtime( $srcFilePathItem );
					if ( $mtime > $lastmtime )
					{
						$srcFilePath = $srcFilePathItem;
						$lastmtime = $mtime;
					}
				}
			}
			else
			{
				// 通常モードの場合、tmpファイルを覗いたファイルを該当ファイルとする。（該当ファイルは１つしか存在しないはず）
				foreach ( $srcFilePathArr as $srcFilePathItem )
				{
					if ( getfileext( $srcFilePathItem ) != 'tmp'  )
					{
						$srcFilePath = $srcFilePathItem;
						break;
					}
				}
			}
			
			// ブラウザキャッシュとの比較（If-Modified-Since対応）
			//$request_headers = apache_request_headers();
			$etag = md5( $entryName . $entryId . $fileid );	// Etagは、entryname+entryid+fileidから生成
			$timestamp = filemtime( $srcFilePath );
			$sLastModified = gmdate('D, d M Y H:i:s T', $timestamp);
			$modified = true;	// 初期値
			
			// ブラウザキャッシュより実画像の更新日時の方が古ければ、$modifiedをfalse（未更新）にする。
			if( isset( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ) ) {
				$since = parse_http_date( $_SERVER['HTTP_IF_MODIFIED_SINCE'] );
				if( $since["timestamp"] >= $timestamp ) {
					$modified = false;
				}
			}
			
			// 画像が未更新なら、304 Not Modifiedを返して終了
			if ( ! $modified )
			{
					header( "HTTP/1.1 304 Not Modified" );
					header( "Etag: \"$etag\"" );
					return;		// ここで終了
			}
			
			// ファイル拡張子の取得
			$fileext = getfileext( $srcFilePath );
			
			// 拡張子が tmp の場合、元ファイル名は取得できていないはずなので、ここで仮の名前をつける。
			if ( $fileext == 'tmp' )
			{
				$orgFileName = 'untitled.' . gettmpfileext( $srcFilePath );
			}
			
			// ダウンロード開始
			mb_http_output("pass");
			
			$mimetypes = array(
				// html
				'css'	=>	'text/css',
				'htm'	=>	'text/html',
				'html'	=>	'text/html',
				'xhtml'	=>	'application/xhtml+xml',
				'xml'	=>	'text/xml',
				'js'	=>	'text/javascript',
				'vbs'	=>	'text/vbscript',
				'rss'	=>	'application/rss+xml',
				'swf'	=>	'application/x-shockwave-flash',
				
				// documents
				'txt'	=>	'text/plain',
				'pdf'	=>	'application/pdf',
				'ps'	=>	'application/postscript',
				'doc'	=>	'application/msword',
				'xls'	=>	'application/msexcel',
				'ppt'	=>	'application/mspowerpoint',
				'docx'	=>	'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
				'xlsx'	=>	'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
				'pptx'	=>	'application/vnd.openxmlformats-officedocument.presentationml.presentation',
				'rtf'	=>	'application/rtf',
				
				// images
				'bmp'	=>	'image/bmp',
				'eps'	=>	'application/postscript',
				'gif'	=>	'image/gif',
				'ico'	=>	'image/x-icon',
				'jpeg'	=>	'image/jpeg',
				'jpg'	=>	'image/jpeg',
				'png'	=>	'image/png',
				'tif'	=>	'image/tiff',
				'tiff'	=>	'image/tiff',
				
				// audio
				'mp3'	=>	'audio/mpeg',
				'mid'	=>	'audio/midi',
				'midi'	=>	'audio/midi',
				'ra'	=>	'audio/vnd.rn-realaudio',
				'rm'	=>	'application/vnd.rn-realmedia',
				'aiff'	=>	'audio/aiff',
				'wav'	=>	'audio/wav',
				'm4a'	=>	'audio/aac',
				'ogg'	=>	'audio/ogg',
				
				// video
				'mpg'	=>	'video/mpeg',
				'mpeg'	=>	'video/mpeg',
				'mov'	=>	'video/quicktime',
				'qt'	=>	'video/quicktime',
				'avi'	=>	'video/x-msvideo',
				'mp4'	=>	'video/mp4',
				'webm'	=>	'video/webm',
				'ogv'	=>	'video/ogg',
				
				// other
				'zip'	=>	'application/zip',
				'OTHER'	=>	'application/octet-stream'	// terminater
			);
			
			// dispが'inline'且つ、対応する拡張子ならば、Content-disposition: inline にし、拡張子に合わせた適切なContent-typeを送る。そうでなければダウンロードさせる。
			if ( $disp == 'inline' && array_key_exists( $fileext, $mimetypes )	){
				$mimetype = $mimetypes[ $fileext ];
				$inline_or_attachment = 'inline';
			}
			else {
				$mimetype = 'application/octet-stream';
				$inline_or_attachment = 'attachment';
			}
			
			// MIMEタイプの送信
			header( 'Content-type: ' . $mimetype );
			
			// SafariかIEならSJISでファイル名を送信する。
			if (preg_match('/\bMSIE\b|\bSafari [12345]\b/', getenv('HTTP_USER_AGENT'))) {
				header('Content-Disposition: ' . $inline_or_attachment . '; filename="' . mb_convert_encoding($orgFileName, 'SJIS', mb_detect_encoding( $orgFileName )) . '"');
			}
			// それ以外のブラウザならUTF-8にしてRFC2231形式でファイル名を送信する。
			else {
				header('Content-Disposition: ' . $inline_or_attachment . '; filename*=UTF-8\'\'' . rawurlencode(mb_convert_encoding( $orgFileName, 'UTF-8', mb_detect_encoding( $orgFileName ))));
			}
			
			header("Content-length: " . filesize( $srcFilePath ));
			//$fp = fopen( $srcFilePath , "rb");
			//print fread($fp, filesize($srcFilePath));
			header('ETag: "' . $etag . '"' );
			header("Last-Modified: " . $sLastModified );
			
			
			// readfileはメモリを大量に消費する為、freadに変更。2009.03.05 jshin
			$fp = fopen( $srcFilePath, 'rb');
			if ( $fp )
			{
				while (!feof($fp))
				{
					echo fread($fp, 4096);
				}
				flush();
				fclose($fp);
			}
			else
			{
				cmsd_abort_proc( 'Can not read the file.' );
			}
			
			//readfile( $srcFilePath );
			//fclose($fp);
		}
	}
	
	// 画像ダウンロード
	class ImageDownloader
	{
		function __construct()
		{
		}
		
		function output()
		{
			// 画像IDパラメータ群の取得
			$entryName = $_GET['entryname'];
			$entryId = $_GET['entryid'];
			$fileid = $_GET['fileid'];
			$mode = getGETParam( 'mode' );	// 'manager'ならば、管理画面モード。
			
			// 編集中のファイルを要求された場合、まだエントリデータ中に情報が保存されてないので、拡張子情報を実際のフォルダのファイルから探しあてる必要がある。
			// 拡張子「*」で検索して、ファイルリストを取得する。
			$srcFilePathDammy = App::createAttachedFilePath( $entryName, $entryId, $fileid, 'dammy.*' );
			$srcFilePathArr = glob( $srcFilePathDammy );
			$srcFilePath = App::createAttachedFilePath( $entryName, $entryId, $fileid, 'nofile.dmy' );
			
			// 対象ファイルを検索
			if ( $mode == 'manager' )
			{
				// 管理画面モードの場合、更新日時が一番新しいファイルを該当ファイルとする。
				$lastmtime = 0;	// 一番新しいファイルの更新日時（初期値は0から開始）
				foreach ( $srcFilePathArr as $srcFilePathItem )
				{
					$mtime = filemtime( $srcFilePathItem );
					if ( $mtime > $lastmtime )
					{
						$srcFilePath = $srcFilePathItem;
						$lastmtime = $mtime;
					}
				}
			}
			else
			{
				// 通常モードの場合、tmpファイルを除いたファイルを該当ファイルとする。（該当ファイルは１つしか存在しないはず）
				foreach ( $srcFilePathArr as $srcFilePathItem )
				{
					if ( getfileext( $srcFilePathItem ) != 'tmp'  )
					{
						$srcFilePath = $srcFilePathItem;
						break;
					}
				}
			}
			
			// ファイル名（サーバに格納されている実際のファイル名）を取得（HTTPヘッダに送信する為）2013/3/12追加
			$srcFileName = basename( $srcFilePath );
			
			// ファイルの拡張子から、HTTPヘッダにつけるMIMEタイプを作る
			$ext = strtolower( getfileext( basename( $srcFilePath ) ) );
			if ( $ext == 'tmp' )
			{
				$ext = strtolower( gettmpfileext( basename( $srcFilePath ) ) );
			}
			
			switch ( $ext )
			{
				case 'jpg' :
					$ext = 'jpeg';
					break;
			}
			
			// 縮小指定パラメータを取得
			$width = null;
			$height = null;
			
			if ( !empty( $_GET['w'] ) ) 
			{ 
				$width = $_GET['w']; 
			}
			
			if ( !empty( $_GET['h'] ) )
			{
				$height = $_GET['h'];
			}
			
			// $width, $heightとは別に、縮小指定パラメータを数値として保持しておく（ファイル名の生成用）。$width, $heightは、「指定されていたかどうか？」の判定に残しておく必要がある。
			$paramWidth = parseNumeric( getGETParam( 'w' ), 0 );
			$paramHeight = parseNumeric( getGETParam( 'h' ), 0 );
			
			// ブラウザキャッシュとの比較（If-Modified-Since対応）
			//$request_headers = apache_request_headers();
			$etag = md5( $entryName . $entryId . $fileid );	// Etagは、entryname+entryid+fileidから生成（現在は、LastModifiedを使って比較している為、Etagは使用していない。指定はダミー。）
			$timestamp = filemtime( $srcFilePath );
			$sLastModified = gmdate('D, d M Y H:i:s T', $timestamp);
			$modified = true;	// 初期値
			
			// ブラウザキャッシュより実画像の更新日時の方が古ければ、$modifiedをfalse（未更新）にする。
			if( isset( $_SERVER['HTTP_IF_MODIFIED_SINCE'] )) {
				$since = parse_http_date( $_SERVER['HTTP_IF_MODIFIED_SINCE'] );
				if( $since["timestamp"] >= $timestamp ) {
					$modified = false;
				}
			}
			
			// 画像が未更新なら、304 Not Modifiedを返して終了
			if ( ! $modified )
			{
					header( "HTTP/1.1 304 Not Modified" );
					header( "Etag: \"$etag\"" );
					return;		// ここで終了
			}
			
			if ( empty($width) and empty($height) )	// 画像縮小指定がない場合
			{
				// ダウンロード開始
				mb_http_output("pass");
	   			header("Content-type: image/" . $ext );
				header("Content-disposition: inline; filename=" . $srcFileName ); // 2013/3/12追加
				header("Content-length: " . filesize( $srcFilePath ));
				header('ETag: "' . $etag . '"' );
				header("Last-Modified: " . $sLastModified );
				//$fp = fopen( $srcFilePath , "r");
   	 			//echo fread($fp, filesize($srcFilePath));
				//fclose($fp);
				
				// readfileはメモリを大量に消費する為、freadに変更。2009.03.05 jshin
				$fp = fopen( $srcFilePath, 'rb');
				if ( $fp )
				{
					while (!feof($fp))
					{
						echo fread($fp, 4096);
					}
					flush();
					fclose($fp);
				}
				else
				{
					cmsd_abort_proc( 'Can not read the file: ' . $srcFileName );
				}
				
				/*
				if ( ! readfile( $srcFilePath ) )
				{
					cmsd_abort_proc( 'Can not read the file.' );
				}
				*/
			}
			else	// 画像縮小指定がある場合
			{
				// キャッシュ画像ファイルパスの生成 2013/3/13 cmsdesigner/cache/imagecache.entryname1.00001.00000001.120x200.gif
				$fileNameImageCache = App::createImageCacheFileName( $entryName, $entryId, $fileid, $paramWidth, $paramHeight, $srcFileName );
				$filePathImageCache = App::createImageCacheFilePath( $entryName, $entryId, $fileid, $paramWidth, $paramHeight, $srcFileName );
					
				// キャッシュが存在するなら、キャッシュを読み込む（存在しなければ縮小処理へ進む）
				if ( $mode != 'manager' )	// 管理モードの場合はキャッシュは存在しないので除外
				{
					/*
					// ファイルパスの生成（旧）cmsdesigner/cache/imagecache.entryname1.00001.00000001.gif.120x200
					$paramWidth = parseNumeric( getGETParam( 'w' ), 0 );
					$paramHeight = parseNumeric( getGETParam( 'h' ), 0 );
					$fileNameImageCache = App::createAttachedFileName( $entryName, $entryId, $fileid, $srcFilePath ) . ".{$paramWidth}x{$paramHeight}";
					$filePathImageCache = App::getRootDir() . "/cache/imagecache." . $fileNameImageCache;
					*/
					if ( file_exists( $filePathImageCache ) )
					{
						// ダウンロード開始
						mb_http_output("pass");
						header("Content-type: image/" . $ext );
						header("Content-disposition: inline; filename=" . $fileNameImageCache ); // 2013/3/12追加
						header("Content-length: " . filesize( $filePathImageCache ));
						header('ETag: "' . $etag . '"' );
						header("Last-Modified: " . $sLastModified );

						// readfileはメモリを大量に消費する為、freadに変更。2013.03.13
						// readfile( $filePathImageCache );
						$fp = fopen( $filePathImageCache, 'rb');
						if ( $fp )
						{
							while (!feof($fp))
							{
								echo fread($fp, 4096);
							}
							flush();
							fclose($fp);
						}
						else
						{
							cmsd_abort_proc( 'Can not read the file: ' . $fileNameImageCache );
						}
						
						return; // outputメソッドを強制終了する
					}
				
					// キャッシュ対象かどうか？
					// Configの読み込み
					$siteConfig = new CMSSiteConfig();
					$entryConfig = $siteConfig->getEntryConfig( $entryName );
					$isCacheTarget = false;
					foreach ( $entryConfig->imageCacheList as $imageCache )
					{
						if ( $imageCache->isTarget( $paramWidth, $paramHeight ) )
						{
							$isCacheTarget = true;
							break;
						}
					}
				}
				
				// ここから、キャッシュが無い場合の画像縮小処理
				
				// ソース画像の大きさを取得
				$info = getimagesize( $srcFilePath );
				$widthSrc = $info[0];
				$heightSrc = $info[1];
				
				// 指定された幅、又は高さに合わせた新しいサイズを計算
				/// 高さと幅を同時に指定されていたら、縮小率が高い方（値が小さい方）を優先する（領域内に収まるようにする）
				if ( !empty($width) and !empty($height) )
				{
					if ( ( $width / $widthSrc ) >= ( $height / $heightSrc ) )
					{
						unset( $width );
					}
					else
					{
						unset( $height );
					}
				}
				
				/// 縮小後のサイズを決定
				if (!empty($width))	// widthが指定されていたら
				{
					$widthNew = $width;
					$heightNew = ($width * $heightSrc) / $widthSrc;
				}
				else
				{
					$heightNew = $height;
					$widthNew = ($height * $widthSrc) / $heightSrc;
				}
				
				// ソース画像を取得
				switch( $info[2] )
				{
					case 1: // GIF
						$imageSrc = imagecreatefromgif( $srcFilePath );
						break;
					
					case 2: // JPEG
						$imageSrc = imagecreatefromjpeg( $srcFilePath );
						break;
					
					case 3: // PNG
						$imageSrc = imagecreatefrompng( $srcFilePath );
						break;
				}
				
				// 出力画像を生成
				if ( function_exists( 'imagecreatetruecolor' ) )
				{
					$imageNew = imagecreatetruecolor( $widthNew, $heightNew );
				}
				else
				{
					$imageNew = imagecreate( $widthNew, $heightNew );
				}
				
				// 出力画像に、ソース画像を縮小して描画
				global $gGDimagecopyfunc;
				if ( $gGDimagecopyfunc === 1 and function_exists( 'imagecopyresampled' ) )
				{
					imagecopyresampled( $imageNew, $imageSrc, 0, 0, 0, 0, $widthNew, $heightNew, $widthSrc, $heightSrc );
				}
				else
				{
					imagecopyresized( $imageNew, $imageSrc, 0, 0, 0, 0, $widthNew, $heightNew, $widthSrc, $heightSrc );
				}
				header("Content-type: image/" . $ext );
				header("Content-disposition: inline; filename=" . $fileNameImageCache ); // 2013/3/13追加
				header('ETag: "' . $etag . '"' );
				header("Last-Modified: " . $sLastModified );
				
				// 元画像リソースを解放
				imagedestroy($imageSrc);
				
				// 画像を出力
				global $gGDJpegQuality;
				switch( $info[2] )
				{
					case 1: // GIF
						imagegif( $imageNew );
						break;
					
					case 2: // JPEG
						imagejpeg( $imageNew, null, $gGDJpegQuality );
						break;
					
					case 3: // PNG
						imagepng( $imageNew );
						break;
				}
				
				// キャッシュ対象の場合、キャッシュを生成する
				if ( $mode != 'manager' and $isCacheTarget )
				{
					$adimageCache = new ADImage();
					$adimageCache->loadFromResourceID( $imageNew );
					/*
					// ファイルパスの生成 cmsdesigner/cache/imagecache.entryname1.00001.00000001.gif.120x200
					// ※ $entryName, $entryId, $fileid はこの関数の最初の方で安全チェック済み。
					$ext = strtolower( getfileext( basename( $srcFilePath ) ) );
					$width = parseNumeric( getGETParam( 'w' ), 0 );
					$height = parseNumeric( getGETParam( 'h' ), 0 );
					$filePathImageCache = App::getRootDir() . "/cache/imagecache.{$entryName}.{$entryId}.{$fileid}.{$ext}.{$width}x{$height}";
					*/
					$adimageCache->saveToFile( $filePathImageCache, $info[2] );
					// サーバによってはファイル所有者apacheで0600になってしまう場合があり、FTPでGETできないので、ここでパーミッションだけでも0644に設定する。
					chmod( $filePathImageCache, 0644 ); 
				}
				
				// 新画像リソースを破棄
				imagedestroy($imageNew);
			}
		}
	}
	
	
	
	/* ---------------------------------------------------------------
	 * 外部XML用ビュークラス。
	 * 
	 */
	class externalxml_view
	{
		var $src = '';
		
		/* ---------------------------------------------------------------
		 * コンストラクタ。
		 * $src : 外部XMLのURL、又はパス名。
		 */
		function __construct( $src )
		{
			$this->src = $src;
			
		}
		
		
		/* ---------------------------------------------------------------
		 * データ無しの場合の結果DOMツリーを生成する。
		 * $message : メッセージ。
		 */
		function createNodataResult( $message = '' )
		{
			// 空っぽのエントリデータ「nodata」要素を生成して返却する。
			
			// DOMをを生成。
			$xmlDoc = domxml_new_doc( '1.0' );
			
			// ルートノード'nodata'を生成。
			$xmlRoot = $xmlDoc->add_root( 'nodata' );
			
			// メッセージが指定されていれば、メッセージ要素を追加する。
			if ( strlen( $message ) > 0 )
			{
				$xmlMessage = $xmlDoc->create_element( 'message' );
				$xmlMessage->set_content( fileconv( $message ) );
				$xmlRoot->append_child( $xmlMessage );
			}
			
			return $xmlDoc;
		
		}
		
		
		/* ---------------------------------------------------------------
		 * エントリリストの表示。
		 * $designName : XSLスタイルシート名。
		 * $isXML : XMLとして出力するか。 (省略時はfalse)
		 * $dateInfoAddition : 日付情報を出力に追加するか。
		 */
		//function listpage( $entryName, $designName = null, $pageMaxRows = 10, $groupKeyValue = '', $showNavi = true, $pageNo = NULL, $isRSS = false, $isXML = false )
		function output( $designName = null, $isXML = false, $dateInfoAddition = true )
		{
			$src = $this->src;
			
			// スキーマ名は"xml"固定。
			$schemaName = 'xml';

			// 指定のデザインファイルが存在しない場合、エラーメッセージを表示して終了する。
			if ( ! $isXML || $designName )	// XML指定じゃない場合は無条件でチェック、designNameが指定されていれば無条件でチェック。
			{
				if ( ! file_exists( App::createXslFilePath( $schemaName, $designName ) ))
				{
					return "ファイル'" . App::createXslFileName( $schemaName, $designName ) . "'が見つかりません。又は埋め込みコマンドのデザイン名の指定が間違っています。";
				}
			}
			
			// srcが、ローカルファイルか、またはリモートファイルか
			$isRemote = false;
			if ( substr( $src, 0, 7 ) == 'http://' )	// http://で始まるsrcの場合、リモートファイルと見なす。
			{
				$isRemote = true;
			}
			
			// 外部XMLをdomとして取得する。
			$xmlDocSrc = null;
			
			if ( $isRemote )
			{
				// リモートファイルの場合、fopenを使って少しずつ読み込み、一つの文字列に結合する。
				// URL先のページを読み込む。 
				$fp = @fopen( $src, 'r' ); // @をつけることでエラー表示を抑制する。
				
				if ( empty( $fp ) ) 
				{
					$xmlDocSrc = $this->createNodataResult( '指定された外部XMLのURLからのデータ読み込みに失敗しました。src=' . $src );
				}
				else
				{
				
					// 少しずつデータを読み込んで、一つの文字列に結合する。
					$strXmlSrc = '';
					while( !feof( $fp ) ) 
					{
					  $strXmlSrc .= fgets( $fp, 4096 );
					}
					
					// 読み込み終了
					fclose( $fp );
					
					// DOMの生成
					$errors = array();
					$xmlDocSrc = @domxml_open_mem( $strXmlSrc, DOMXML_LOAD_PARSING, $errors );
					
					if ( is_null( $xmlDocSrc ) )
					{
						$xmlDocSrc = $this->createNodataResult( '指定された外部XMLデータは適切なXMLデータではありません。src=' . $src );
					}
					elseif ( count( $errors ) > 0 )
					{
						$xmlDocSrc = $this->createNodataResult( $errors[0]['errormessage'] . ', src=' . $src );
					}
					
				}
				
			}
			else
			{
				// ローカルファイルの場合、domxml_open_fileを使って読み込む。
				if ( file_exists( $src ) )
				{
					// ファイルを読み込む。
					$errors = array();
			   	 	$xmlDocSrc = @domxml_open_file( $src, DOMXML_LOAD_PARSING, $errors );
					
					if ( is_null( $xmlDocSrc ) )
					{
						$xmlDocSrc = $this->createNodataResult( '指定された外部XMLデータは適切なXMLデータではありません。src=' . $src );
					}
					elseif ( count( $errors ) > 0 )
					{
						$xmlDocSrc = $this->createNodataResult( $errors[0]['errormessage'] . ', src=' . $src );
					}
				}
				else
				{
					$xmlDocSrc = $this->createNodataResult( '指定された外部XMLは存在しません。src=' . $src ); 
				}
				
			}
			
			// CMSD独自の日付情報を埋め込む。
			if ( $dateInfoAddition )
			{
				$xmlSrcRoot = $xmlDocSrc->document_element();
				$rootTagName = $xmlSrcRoot->tagname();
				
				// 2007.02.27 excite blogのように、RSS2.0にも関わらずdc:dateで日時情報を出力しているfeedが存在する為、両方対処するようにした。
				switch ( $rootTagName )
				{
					case 'RDF' :	// RSS1.0
					case 'rdf:RDF' :	// for PHP5
					case 'rss' :	// RSS0.91 or RSS2.0
					
						// XPathを準備
						$xpath = $xmlDocSrc->xpath_new_context();
						xpath_register_ns( $xpath, 'dc', 'http://purl.org/dc/elements/1.1/');
						xpath_register_ns( $xpath, 'rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' );
						
						// RSS1.0用の処理
						$xmlDateArr = $xpath->xpath_eval('//*[self::dc:date]');//( '//dc:date' ); // '//dc:date'だと、XREAサーバーのPHP5.3環境でハングアップする為、この表記に変えた。なぜハングアップするかは不明。namespaceに関係しているライブラリのバグの可能性が高い。
						
						if ( !empty($xmlDateArr->nodeset) )
						{
							foreach( $xmlDateArr->nodeset as $xmlDate )
							{
								$strDate = $xmlDate->get_content();
								$timestamp = $this->parseDateTime( $strDate, 'W3C/Date' );
								$this->addDateInfo( $xmlDate, $timestamp );
							}
						}
						
						// RSS2.0用の処理
						$xmlDateArr = $xpath->xpath_eval( '//*[self::pubDate or self::lastBuildDate]' );
						
						if ( !empty($xmlDateArr->nodeset) )
						{
							foreach( $xmlDateArr->nodeset as $xmlDate )
							{
								$strDate = $xmlDate->get_content();
								$timestamp = $this->parseDateTime( $strDate, 'RFC822' );
								$this->addDateInfo( $xmlDate, $timestamp );
							}
						}
					
						break;
					
				}
				
				/*
				switch ( $rootTagName )
				{
					case 'RDF' :	// RSS1.0
						$xpath = $xmlDocSrc->xpath_new_context();
						xpath_register_ns( $xpath, 'dc', 'http://purl.org/dc/elements/1.1/');
						xpath_register_ns( $xpath, 'rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' );
						
						$xmlDateArr = $xpath->xpath_eval( '//dc:date' );
						
						if ( $xmlDateArr->nodeset )
						{
							foreach( $xmlDateArr->nodeset as $xmlDate )
							{
								$strDate = $xmlDate->get_content();
								$timestamp = $this->parseDateTime( $strDate, 'W3C/Date' );
								$this->addDateInfo( $xmlDate, $timestamp );
							}
						}
						
						
						break;
					
					case 'rss' :	// RSS0.91 or RSS2.0
						$xpath = $xmlDocSrc->xpath_new_context();
						$xmlDateArr = $xpath->xpath_eval( '//*[self::pubDate or self::lastBuildDate]' );
						
						if ( $xmlDateArr->nodeset )
						{
							foreach( $xmlDateArr->nodeset as $xmlDate )
							{
								$strDate = $xmlDate->get_content();
								$timestamp = $this->parseDateTime( $strDate, 'RFC822' );
								$this->addDateInfo( $xmlDate, $timestamp );
							}
						}
					
						break;
					
				}
				*/
				
			}
			
			
			// XSL変換処理
			if ( $isXML )
			{
				// XMLのまま出力。内部的にはEUC-JPで処理するが、XMLを生で出力したい場合、UTF-8出力になることがほとんどと思われる為、例外的にUTF-8出力とする。
				$result = $xmlDocSrc->dump_mem( true, 'UTF-8' );
				$result = trim( $result );	// 2006.11.15 追加：XMLツリー取得結果の前後に余計な改行が追加される為、強制的に前後の改行や空白を除去。
				
			}
			// 通常のページ表示変換処理
			else
			{
				$result = ADXslt::process( $xmlDocSrc, App::createXslFilePath( $schemaName, $designName) );
			}
			return $result;
			
		}
		
		
		function addDateInfo( &$xmlDate, $timestamp )
		{
			// 日時情報をローカル日時で取得

			$sYear = date( 'Y', $timestamp );
			$sMonth = date( 'm', $timestamp );
			$sDay = date( 'd', $timestamp );
			$sWeekday = strtolower( date( 'D', $timestamp ) );
			$sHour = date( 'H', $timestamp );
			$sMinute = date( 'i', $timestamp );
			$sSecond = date( 's', $timestamp );
			$sTimezone = date( 'O', $timestamp );
			
			// 日付要素に各情報を属性として追加
			
			$xmlDate->set_attribute( 'year', $sYear );
			$xmlDate->set_attribute( 'month', $sMonth );
			$xmlDate->set_attribute( 'day', $sDay );
			$xmlDate->set_attribute( 'weekday', $sWeekday );
			$xmlDate->set_attribute( 'hour', $sHour );
			$xmlDate->set_attribute( 'minute', $sMinute );
			$xmlDate->set_attribute( 'second', $sSecond );
			
			// テスト
			$xmlDate->set_attribute( 'timezone', $sTimezone );
			
		}
		
		function parseDateTime( $strDate, $type )
		{
			$timestamp = null;	// 初期化
			
			switch ( $type )
			{
				case 'W3C/Date' :
					// セパレータ'T'で、日付部分と時刻部分を分離する。
					$arrDateToken = explode( 'T', $strDate );
					
					// 日付用変数の初期化
					$year = 0;
					$month = 0;
					$day = 0;
					
					// 日付部分の分解
					if ( isset( $arrDateToken[0] ) )
					{
						$year = substr( $arrDateToken[0], 0, 4 );
						$month = substr( $arrDateToken[0], 5, 2 );
						$day = substr( $arrDateToken[0], 8, 2 );
					}
					
					// 時刻と時差用変数の初期化
					$hour = 0;
					$minute = 0;
					$second = 0;
					$milisecond = 0;
					$offset_sign = '';	// + or - 記号
					$offset_hour = 0;
					$offset_minute = 0;
					
					// 時刻部分が存在するか？
					if ( isset( $arrDateToken[1] ) )
					{
						// 時刻部分を、さらにローカル時刻と時差に分離する。
						$arrDateToken2 = preg_split( '/[-+Z]/', $arrDateToken[1] );	// 最初、[+-Z]と書いたら、どうやら"-Z"で文字集合として扱われた(?)らしく、マイナスが認識されなかった。
						
						// ローカル時刻部分の分解
						if ( isset( $arrDateToken2[0] ) )
						{
							$hour = substr( $arrDateToken2[0], 0, 2 );
							$minute = substr( $arrDateToken2[0], 3, 2 );
							$second = substr( $arrDateToken2[0], 6, 2 );
							$milisecond = substr( $arrDateToken2[0], 9 );
						}
						
						// 時差符号の取得：ローカル時刻文字列の長さ+1の文字が、時差符号。
						$offset_sign = substr( $arrDateToken[1], strlen( $arrDateToken2[0] ), 1 );
						
						// 時差部分の分解
						if ( isset( $arrDateToken2[1] ) )
						{
							$offset_hour = substr( $arrDateToken2[1], 0, 2 );
							$offset_minute = substr( $arrDateToken2[1], 3, 2 );
							
							// 時差が'Z'とかかれていたら、"+00:00"の意味。
							if ( $offset_sign == 'Z' )
							{
								$offset_sign = '+';
								$offset_hour = '00';
								$offset_minute = '00';
							}
						}
					}
					
					// GMT基準でのtimestampを取得する。
					$timestamp = gmmktime( $hour, $minute, $second, $month, $day, $year );
					
					// 時差を加える。
					$offset_time = $offset_hour * 3600 + $offset_minute * 60;
					if ( $offset_sign == '-' )
					{
						$offset_time *= -1;
					}
					$timestamp -= $offset_time;	// 時差が+09:00なら、9時間分を引けば、GMTでの正しいtimestampが算出できる。
					
					break;
					
				
				case 'RFC822' :
					
					$timestamp = strtotime( $strDate );
				
					break;
			
			}

			
			return $timestamp;
		}
		
	}
	
?>