<?php
	// このファイルはGoogle Maps API v3対応版です。
	// v2対応版が必要な場合には、inputgmap_v2.phpをご利用下さい。

	require_once( dirname(__FILE__) . "/include/app.php.inc" );
	
	// 設定ファイルから規定値を読み込む
	// Site.Configより指定されたエントリ設定情報を取得する。
	$defaultLat = "35.681382";
	$defaultLng = "139.76608399999998";
	$defaultZoom = "15";
	$gmapKey = "";
	$xmlDocSite = domxml_open_file( App::getSiteConfigFilePath() );
	$xpath = $xmlDocSite->xpath_new_context();

	$xmlGMapConfigArr = $xpath->xpath_eval( "/site/gmap" );
	if ( count($xmlGMapConfigArr->nodeset) > 0 )
	{
		$xmlGMapConfig = $xmlGMapConfigArr->nodeset[0];
		$gmapKey = nullString( $xmlGMapConfig->get_attribute("key") );
		$defaultLat = nullString( $xmlGMapConfig->get_attribute("lat"), $defaultLat );
		$defaultLng = nullString( $xmlGMapConfig->get_attribute("lng"), $defaultLng );
		$defaultZoom = nullString( $xmlGMapConfig->get_attribute("zoom"), $defaultZoom );
	}
	
	// 各パラメータの取得
	$lat = getGETParam( 'lat' , '' );
	$lng = getGETParam( 'lng', '' );
	$zoom = getGETParam( 'zoom', '' );
	$targetItemName = getGETParam( 'tname', '' );
	
	// latが数値でなかった場合は空にする
	if ( $lat !== '' and ! is_numeric( $lat ) )
	{
		$lat = '';
	}
	
	// lngが数値でなかった場合は空にする。
	if ( $lng !== '' and ! is_numeric( $lng ) )
	{
		$lng = '';
	}
	
	// zoomが数値でなかった場合は空にする。
	if ( $zoom !== '' and ! is_numeric( $zoom ) )
	{
		$zoom = '';
	}
	
	// latかlngが空だったら、規定値を入れる。
	if ( $lat === '' or $lng === '' )
	{
		$lat = $defaultLat;
		$lng = $defaultLng;
	}
	
	// zoomが空だったら、規定値を入れる。
	if ( $zoom === '' )
	{
		$zoom = $defaultZoom;
	}
	
	// targetItemNameパラメータの必須チェック
	if ( $targetItemName === '' )
	{
		die( 'Error : Bad request' );
	}
	
	$result = "{ 'lat' : $lat, 'lng' : $lng, 'zoom' : $zoom, 'targetItemName' : '$targetItemName' }";

?><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
	<meta http-equiv="content-type" content="text/html; charset=utf-8"/>
	<title>位置情報の入力</title>
	<style tyhpe="text/css">
	<!--
		form {
			padding : 0px;
			margin :0px;
		}
		
		body {
			margin :0px;
			padding :0px;
			margin-top : 20px;
		}
		
		#panelMain {
			width : 550px;
			height : 500px;
			margin-left : auto;
			margin-right : auto;
			margin-bottom : auto;
		}
		
		#panelMap {
			border-left : 1px solid #eeeeee;
			border-top : 1px solid #ddddff;
			border-right : 1px solid #888888;
			border-bottom : 1px solid #888888;
			/* border : 1px solid #88ddff; */
			background-color : #dddddd;
			padding : 24px;
		}
		
		#map {
			width: 500px; height: 300px;
		}
		
		#msg {
			padding:0.2em;
			background-color:#f0f0ff;
			color=red;
		}
		
		#results {
			font-size : 8pt;
			color : #3355ff;
			background-color : #ddeeff;
			padding : 0.5em;
		}
		
		#panelButtons {
			text-align : center;
			padding-top : 10px;
		}
		
		#panelButtons input {
			font-weight : bold;
			margin-left : 10px;
			padding : 0.5em 2em 0.5em 2em;
			background-color : #aabbcc
		}
		
	-->
	</style>
	<script type="text/javascript" src="http://maps.google.com/maps/api/js?v=3&key=<?php print( $gmapKey ); ?>"></script>
	<script type="text/javascript">
	//<![CDATA[
	
	var map = null;
	var sightMarker = null;
	var geocoder = null;
	var geocodedLatLng = null;
	var result = <?php print $result; ?>; //{ "lat" : 35.681391, "lng" : 139.766103, "zoom" : 15, "targetItemName" : "location" };
	
	function load() {
		// マップの生成と初期値設定
		var latlng = new google.maps.LatLng( result.lat, result.lng );
		var myOptions = {
			zoom: result.zoom,
			center: latlng,
			mapTypeId: google.maps.MapTypeId.ROADMAP,
			navigationControl : true,
			navigationControlOptions: { "style" : google.maps.NavigationControlStyle.SMALL },
			scaleControl: false,
			streetViewControl: false
		};
		
		map = new google.maps.Map( document.getElementById( "map" ), myOptions );
		
		// ジオコーダーの生成
		geocoder = new google.maps.Geocoder();
		
		// 照準画像を生成してセットする
		var sightImage = new google.maps.MarkerImage( 
			"./manager.resources/sight.png", 
			new google.maps.Size( 100, 100 ),
			new google.maps.Point( 0, 0 ),
			new google.maps.Point( 49, 49 ) 
		);
		// 照準画像からマーカーを生成し、マップにオーバーレイする
		sightMarker = new google.maps.Marker( { "position": latlng, "map":map, "icon": sightImage } );
		
		// マップのcenter_changedイベントハンドラ
		google.maps.event.addListener( map, "center_changed", function(){
			// 照準マーカーを地図の中心に移動する
			sightMarker.setPosition( map.getCenter() );
			
			// 緯度・経度情報を取得して画面に表示する
			showLatLng( map.getCenter() );
			
			// 住所入力欄をクリアする（住所検索直後は除く）
			if ( ! map.getCenter().equals( geocodedLatLng ) )
			{
				setAddress( "" );
			}
			
			// resultに現在の地図の中心位置を保存する
			result.lat = map.getCenter().lat();
			result.lng = map.getCenter().lng();
		});
		
		// マップのzoom_changedイベントハンドラ
		google.maps.event.addListener( map, "zoom_changed", function(){
			// ズームを取得して画面に表示する
			showZoom( map.getZoom() );
			
			// resultに現在の地図のズームを保存する
			result.zoom = map.getZoom();
		});
		
		// マップのclickイベントハンドラ
		google.maps.event.addListener( map, "click", function( event ){
			// クリックした位置へマップの中心を移動する
			map.panTo( event.latLng );
		});
	
		// CMS Designerから渡された位置とズームを現在の位置とズームに設定する
		map.setCenter( new google.maps.LatLng( result.lat, result.lng, false ) );
		map.setZoom( result.zoom );
		
		return true;
	
	}
	
	function showAddress(address) {
		if (geocoder) {
			// 入力された住所でジオコードを実行
			geocoder.geocode( { "address": address }, function( georesults, geostatus ){
					// 戻り値がOKならば
					if ( geostatus ==  google.maps.GeocoderStatus.OK ) {
						// 返却情報（複数）の先頭の位置をgeocodedLatLngに保存して地図に反映する。
						geocodedLatLng = georesults[0].geometry.location;
						map.setCenter( geocodedLatLng);
						map.setZoom( 15 );
						showMessage( "" );	// メッセージのクリア
						showLatLng( geocodedLatLng );	// 位置情報の表示
						showZoom( map.getZoom() );		// ズーム情報の表示
					}
					// 戻り値がOK以外ならば
					else {
						// エラーメッセージの表示
						showMessage( address + "は見つかりませんでした" );
					}
				}
			);
		}
	
	}
	
	function showMessage( message )
	{
		document.getElementById("msg").innerHTML = message;
	}
	
	function setAddress( newValue)
	{
		document.getElementById( "address" ).value = newValue;
	}
	
	function showLatLng( point )
	{
		document.getElementById( "LatLng" ).innerHTML = point; //"(" + point.lat() + "," + point.lng() + ")";
	}
	
	function showZoom( zoom )
	{
		document.getElementById( "zoom" ).innerHTML = zoom;
	}
	
	function cancelGMap()
	{
		window.close();
	}
	
	function applyGMap()
	{
		//alert( 'lat : ' + result.lat + ', lng : ' + result.lng + ', zoom : ' + result.zoom );
		window.opener.document.frmEntryEditor[ result.targetItemName + '_LAT' ].value = result.lat;
		window.opener.document.frmEntryEditor[ result.targetItemName + '_LNG' ].value = result.lng;
		window.opener.document.frmEntryEditor[ result.targetItemName + '_ZOOM' ].value = result.zoom;
		window.close();
   }

	//]]>
	</script>
  </head>
  <body onload="load();" onunload="GUnload()">
	<div id="panelMain">
		<div id="panelMap">
			<form onsubmit="return false;">
			住所を入力して場所を検索することができます。<br />
			<span style="font-size:0.7em">（入力例：「福井駅」「福井県鯖江市上鯖江1丁目」等）</span><br />
			<input type="text" id="address" name="address" size="60" value="" />
			<input type="button" value="検索" onclick="showAddress(this.form.address.value); return false;" /><br />
			<br />
			十字が示す位置をマウスで動かして位置を指定できます。<br />
			<span style="font-size:0.7em">（※地図が灰色表示の場合は画像の読込中です。しばらくお待ち下さい）</span>
			</form>
			<div id="msg"></div>
			<div id="map"></div>
			<div id="results">
			緯度・経度：<span id="LatLng"></span><br />
			縮尺：<span id="zoom"></span><br />
			</div>
		</div>
		<div id="panelButtons">
			<form onsubmit="return false;">
				<input type="button" value="キャンセル" onclick="cancelGMap();return false;" />
				<input type="button" value="この位置で決定" onclick="applyGMap();return false;" />
			</form>
		</div>
	</div>
  </body>
</html>